trp_player = 0
trp_multiplayer_profile_troop_male = 1
trp_multiplayer_profile_troop_female = 2
trp_temp_troop = 3
trp_find_item_cheat = 4
trp_random_town_sequence = 5
trp_tournament_participants = 6
trp_tutorial_maceman = 7
trp_tutorial_archer = 8
trp_tutorial_swordsman = 9
trp_novice_fighter = 10
trp_regular_fighter = 11
trp_veteran_fighter = 12
trp_champion_fighter = 13
trp_arena_training_fighter_1 = 14
trp_arena_training_fighter_2 = 15
trp_arena_training_fighter_3 = 16
trp_arena_training_fighter_4 = 17
trp_arena_training_fighter_5 = 18
trp_arena_training_fighter_6 = 19
trp_arena_training_fighter_7 = 20
trp_arena_training_fighter_8 = 21
trp_arena_training_fighter_9 = 22
trp_arena_training_fighter_10 = 23
trp_cattle = 24
trp_farmer = 25
trp_townsman = 26
trp_watchman = 27
trp_caravan_guard = 28
trp_mercenary_swordsman = 29
trp_hired_blade = 30
trp_mercenary_crossbowman = 31
trp_mercenary_horseman = 32
trp_mercenary_cavalry = 33
trp_mercenaries_end = 34
trp_swadian_recruit = 35
trp_swadian_militia = 36
trp_swadian_footman = 37
trp_swadian_infantry = 38
trp_swadian_sergeant = 39
trp_swadian_skirmisher = 40
trp_swadian_crossbowman = 41
trp_swadian_sharpshooter = 42
trp_swadian_man_at_arms = 43
trp_swadian_knight = 44
trp_swadian_messenger = 45
trp_swadian_deserter = 46
trp_swadian_prison_guard = 47
trp_swadian_castle_guard = 48
trp_vaegir_recruit = 49
trp_vaegir_footman = 50
trp_vaegir_skirmisher = 51
trp_vaegir_archer = 52
trp_vaegir_marksman = 53
trp_vaegir_veteran = 54
trp_vaegir_infantry = 55
trp_vaegir_guard = 56
trp_vaegir_horseman = 57
trp_vaegir_knight = 58
trp_vaegir_messenger = 59
trp_vaegir_deserter = 60
trp_vaegir_prison_guard = 61
trp_vaegir_castle_guard = 62
trp_khergit_tribesman = 63
trp_khergit_skirmisher = 64
trp_khergit_horseman = 65
trp_khergit_horse_archer = 66
trp_khergit_veteran_horse_archer = 67
trp_khergit_lancer = 68
trp_khergit_messenger = 69
trp_khergit_deserter = 70
trp_khergit_prison_guard = 71
trp_khergit_castle_guard = 72
trp_nord_recruit = 73
trp_nord_footman = 74
trp_nord_trained_footman = 75
trp_nord_warrior = 76
trp_nord_veteran = 77
trp_nord_champion = 78
trp_nord_huntsman = 79
trp_nord_archer = 80
trp_nord_veteran_archer = 81
trp_nord_messenger = 82
trp_nord_deserter = 83
trp_nord_prison_guard = 84
trp_nord_castle_guard = 85
trp_rhodok_tribesman = 86
trp_rhodok_spearman = 87
trp_rhodok_trained_spearman = 88
trp_rhodok_veteran_spearman = 89
trp_rhodok_sergeant = 90
trp_rhodok_crossbowman = 91
trp_rhodok_trained_crossbowman = 92
trp_rhodok_veteran_crossbowman = 93
trp_rhodok_sharpshooter = 94
trp_rhodok_messenger = 95
trp_rhodok_deserter = 96
trp_rhodok_prison_guard = 97
trp_rhodok_castle_guard = 98
trp_sarranid_recruit = 99
trp_sarranid_footman = 100
trp_sarranid_veteran_footman = 101
trp_sarranid_infantry = 102
trp_sarranid_guard = 103
trp_sarranid_skirmisher = 104
trp_sarranid_archer = 105
trp_sarranid_master_archer = 106
trp_sarranid_horseman = 107
trp_sarranid_mamluke = 108
trp_sarranid_messenger = 109
trp_sarranid_deserter = 110
trp_sarranid_prison_guard = 111
trp_sarranid_castle_guard = 112
trp_looter = 113
trp_bandit = 114
trp_brigand = 115
trp_mountain_bandit = 116
trp_forest_bandit = 117
trp_sea_raider = 118
trp_steppe_bandit = 119
trp_taiga_bandit = 120
trp_desert_bandit = 121
trp_black_khergit_horseman = 122
trp_manhunter = 123
trp_slave_driver = 124
trp_slave_hunter = 125
trp_slave_crusher = 126
trp_slaver_chief = 127
trp_follower_woman = 128
trp_hunter_woman = 129
trp_fighter_woman = 130
trp_sword_sister = 131
trp_refugee = 132
trp_peasant_woman = 133
trp_caravan_master = 134
trp_kidnapped_girl = 135
trp_town_walker_1 = 136
trp_town_walker_2 = 137
trp_khergit_townsman = 138
trp_khergit_townswoman = 139
trp_sarranid_townsman = 140
trp_sarranid_townswoman = 141
trp_village_walker_1 = 142
trp_village_walker_2 = 143
trp_spy_walker_1 = 144
trp_spy_walker_2 = 145
trp_tournament_master = 146
trp_trainer = 147
trp_constable_hareck = 148
trp_ramun_the_slave_trader = 149
trp_guide = 150
trp_xerina = 151
trp_dranton = 152
trp_kradus = 153
trp_tutorial_trainer = 154
trp_tutorial_student_1 = 155
trp_tutorial_student_2 = 156
trp_tutorial_student_3 = 157
trp_tutorial_student_4 = 158
trp_galeas = 159
trp_farmer_from_bandit_village = 160
trp_trainer_1 = 161
trp_trainer_2 = 162
trp_trainer_3 = 163
trp_trainer_4 = 164
trp_trainer_5 = 165
trp_ransom_broker_1 = 166
trp_ransom_broker_2 = 167
trp_ransom_broker_3 = 168
trp_ransom_broker_4 = 169
trp_ransom_broker_5 = 170
trp_ransom_broker_6 = 171
trp_ransom_broker_7 = 172
trp_ransom_broker_8 = 173
trp_ransom_broker_9 = 174
trp_ransom_broker_10 = 175
trp_tavern_traveler_1 = 176
trp_tavern_traveler_2 = 177
trp_tavern_traveler_3 = 178
trp_tavern_traveler_4 = 179
trp_tavern_traveler_5 = 180
trp_tavern_traveler_6 = 181
trp_tavern_traveler_7 = 182
trp_tavern_traveler_8 = 183
trp_tavern_traveler_9 = 184
trp_tavern_traveler_10 = 185
trp_tavern_bookseller_1 = 186
trp_tavern_bookseller_2 = 187
trp_tavern_minstrel_1 = 188
trp_tavern_minstrel_2 = 189
trp_tavern_minstrel_3 = 190
trp_tavern_minstrel_4 = 191
trp_tavern_minstrel_5 = 192
trp_kingdom_heroes_including_player_begin = 193
trp_npc1 = 194
trp_npc2 = 195
trp_npc3 = 196
trp_npc4 = 197
trp_npc5 = 198
trp_npc6 = 199
trp_npc7 = 200
trp_npc8 = 201
trp_npc9 = 202
trp_npc10 = 203
trp_npc11 = 204
trp_npc12 = 205
trp_npc13 = 206
trp_npc14 = 207
trp_npc15 = 208
trp_npc16 = 209
trp_kingdom_1_lord = 210
trp_kingdom_2_lord = 211
trp_kingdom_3_lord = 212
trp_kingdom_4_lord = 213
trp_kingdom_5_lord = 214
trp_kingdom_6_lord = 215
trp_knight_1_1 = 216
trp_knight_1_2 = 217
trp_knight_1_3 = 218
trp_knight_1_4 = 219
trp_knight_1_5 = 220
trp_knight_1_6 = 221
trp_knight_1_7 = 222
trp_knight_1_8 = 223
trp_knight_1_9 = 224
trp_knight_1_10 = 225
trp_knight_1_11 = 226
trp_knight_1_12 = 227
trp_knight_1_13 = 228
trp_knight_1_14 = 229
trp_knight_1_15 = 230
trp_knight_1_16 = 231
trp_knight_1_17 = 232
trp_knight_1_18 = 233
trp_knight_1_19 = 234
trp_knight_1_20 = 235
trp_knight_2_1 = 236
trp_knight_2_2 = 237
trp_knight_2_3 = 238
trp_knight_2_4 = 239
trp_knight_2_5 = 240
trp_knight_2_6 = 241
trp_knight_2_7 = 242
trp_knight_2_8 = 243
trp_knight_2_9 = 244
trp_knight_2_10 = 245
trp_knight_2_11 = 246
trp_knight_2_12 = 247
trp_knight_2_13 = 248
trp_knight_2_14 = 249
trp_knight_2_15 = 250
trp_knight_2_16 = 251
trp_knight_2_17 = 252
trp_knight_2_18 = 253
trp_knight_2_19 = 254
trp_knight_2_20 = 255
trp_knight_3_1 = 256
trp_knight_3_2 = 257
trp_knight_3_3 = 258
trp_knight_3_4 = 259
trp_knight_3_5 = 260
trp_knight_3_6 = 261
trp_knight_3_7 = 262
trp_knight_3_8 = 263
trp_knight_3_9 = 264
trp_knight_3_10 = 265
trp_knight_3_11 = 266
trp_knight_3_12 = 267
trp_knight_3_13 = 268
trp_knight_3_14 = 269
trp_knight_3_15 = 270
trp_knight_3_16 = 271
trp_knight_3_17 = 272
trp_knight_3_18 = 273
trp_knight_3_19 = 274
trp_knight_3_20 = 275
trp_knight_4_1 = 276
trp_knight_4_2 = 277
trp_knight_4_3 = 278
trp_knight_4_4 = 279
trp_knight_4_5 = 280
trp_knight_4_6 = 281
trp_knight_4_7 = 282
trp_knight_4_8 = 283
trp_knight_4_9 = 284
trp_knight_4_10 = 285
trp_knight_4_11 = 286
trp_knight_4_12 = 287
trp_knight_4_13 = 288
trp_knight_4_14 = 289
trp_knight_4_15 = 290
trp_knight_4_16 = 291
trp_knight_4_17 = 292
trp_knight_4_18 = 293
trp_knight_4_19 = 294
trp_knight_4_20 = 295
trp_knight_5_1 = 296
trp_knight_5_2 = 297
trp_knight_5_3 = 298
trp_knight_5_4 = 299
trp_knight_5_5 = 300
trp_knight_5_6 = 301
trp_knight_5_7 = 302
trp_knight_5_8 = 303
trp_knight_5_9 = 304
trp_knight_5_10 = 305
trp_knight_5_11 = 306
trp_knight_5_12 = 307
trp_knight_5_13 = 308
trp_knight_5_14 = 309
trp_knight_5_15 = 310
trp_knight_5_16 = 311
trp_knight_5_17 = 312
trp_knight_5_18 = 313
trp_knight_5_19 = 314
trp_knight_5_20 = 315
trp_knight_6_1 = 316
trp_knight_6_2 = 317
trp_knight_6_3 = 318
trp_knight_6_4 = 319
trp_knight_6_5 = 320
trp_knight_6_6 = 321
trp_knight_6_7 = 322
trp_knight_6_8 = 323
trp_knight_6_9 = 324
trp_knight_6_10 = 325
trp_knight_6_11 = 326
trp_knight_6_12 = 327
trp_knight_6_13 = 328
trp_knight_6_14 = 329
trp_knight_6_15 = 330
trp_knight_6_16 = 331
trp_knight_6_17 = 332
trp_knight_6_18 = 333
trp_knight_6_19 = 334
trp_knight_6_20 = 335
trp_kingdom_1_pretender = 336
trp_kingdom_2_pretender = 337
trp_kingdom_3_pretender = 338
trp_kingdom_4_pretender = 339
trp_kingdom_5_pretender = 340
trp_kingdom_6_pretender = 341
trp_knight_1_1_wife = 342
trp_kingdom_1_lady_1 = 343
trp_kingdom_1_lady_2 = 344
trp_knight_1_lady_3 = 345
trp_knight_1_lady_4 = 346
trp_kingdom_l_lady_5 = 347
trp_kingdom_1_lady_6 = 348
trp_kingdom_1_lady_7 = 349
trp_kingdom_1_lady_8 = 350
trp_kingdom_1_lady_9 = 351
trp_kingdom_1_lady_10 = 352
trp_kingdom_1_lady_11 = 353
trp_kingdom_1_lady_12 = 354
trp_kingdom_l_lady_13 = 355
trp_kingdom_1_lady_14 = 356
trp_kingdom_1_lady_15 = 357
trp_kingdom_1_lady_16 = 358
trp_kingdom_1_lady_17 = 359
trp_kingdom_1_lady_18 = 360
trp_kingdom_1_lady_19 = 361
trp_kingdom_1_lady_20 = 362
trp_kingdom_2_lady_1 = 363
trp_kingdom_2_lady_2 = 364
trp_kingdom_2_lady_3 = 365
trp_kingdom_2_lady_4 = 366
trp_kingdom_2_lady_5 = 367
trp_kingdom_2_lady_6 = 368
trp_kingdom_2_lady_7 = 369
trp_kingdom_2_lady_8 = 370
trp_kingdom_2_lady_9 = 371
trp_kingdom_2_lady_10 = 372
trp_kingdom_2_lady_11 = 373
trp_kingdom_2_lady_12 = 374
trp_kingdom_2_lady_13 = 375
trp_kingdom_2_lady_14 = 376
trp_kingdom_2_lady_15 = 377
trp_kingdom_2_lady_16 = 378
trp_kingdom_2_lady_17 = 379
trp_kingdom_2_lady_18 = 380
trp_kingdom_2_lady_19 = 381
trp_kingdom_2_lady_20 = 382
trp_kingdom_3_lady_1 = 383
trp_kingdom_3_lady_2 = 384
trp_kingdom_3_lady_3 = 385
trp_kingdom_3_lady_4 = 386
trp_kingdom_3_lady_5 = 387
trp_kingdom_3_lady_6 = 388
trp_kingdom_3_lady_7 = 389
trp_kingdom_3_lady_8 = 390
trp_kingdom_3_lady_9 = 391
trp_kingdom_3_lady_10 = 392
trp_kingdom_3_lady_11 = 393
trp_kingdom_3_lady_12 = 394
trp_kingdom_3_lady_13 = 395
trp_kingdom_3_lady_14 = 396
trp_kingdom_3_lady_15 = 397
trp_kingdom_3_lady_16 = 398
trp_kingdom_3_lady_17 = 399
trp_kingdom_3_lady_18 = 400
trp_kingdom_3_lady_19 = 401
trp_kingdom_3_lady_20 = 402
trp_kingdom_4_lady_1 = 403
trp_kingdom_4_lady_2 = 404
trp_kingdom_4_lady_3 = 405
trp_kingdom_4_lady_4 = 406
trp_kingdom_4_lady_5 = 407
trp_kingdom_4_lady_6 = 408
trp_kingdom_4_lady_7 = 409
trp_knight_4_2b_daughter_1 = 410
trp_kingdom_4_lady_9 = 411
trp_knight_4_2c_wife_1 = 412
trp_kingdom_4_lady_11 = 413
trp_knight_4_2c_daughter = 414
trp_knight_4_1b_wife = 415
trp_kingdom_4_lady_14 = 416
trp_knight_4_1b_daughter = 417
trp_knight_4_2b_daughter_2 = 418
trp_kingdom_4_lady_17 = 419
trp_knight_4_2c_wife_2 = 420
trp_knight_4_1c_daughter = 421
trp_kingdom_4_lady_20 = 422
trp_kingdom_5_lady_1 = 423
trp_kingdom_5_lady_2 = 424
trp_kingdom_5_lady_3 = 425
trp_kingdom_5_lady_4 = 426
trp_kingdom_5_5_wife = 427
trp_kingdom_5_2b_wife_1 = 428
trp_kingdom_5_1c_daughter_1 = 429
trp_kingdom_5_2c_daughter_1 = 430
trp_kingdom_5_1c_wife_1 = 431
trp_kingdom_5_2c_wife_1 = 432
trp_kingdom_5_1c_daughter_2 = 433
trp_kingdom_5_2c_daughter_2 = 434
trp_kingdom_5_1b_wife = 435
trp_kingdom_5_2b_wife_2 = 436
trp_kingdom_5_1c_daughter_3 = 437
trp_kingdom_5_lady_16 = 438
trp_kingdom_5_1c_wife_2 = 439
trp_kingdom_5_2c_wife_2 = 440
trp_kingdom_5_1c_daughter_4 = 441
trp_kingdom_5_lady_20 = 442
trp_kingdom_6_lady_1 = 443
trp_kingdom_6_lady_2 = 444
trp_kingdom_6_lady_3 = 445
trp_kingdom_6_lady_4 = 446
trp_kingdom_6_lady_5 = 447
trp_kingdom_6_lady_6 = 448
trp_kingdom_6_lady_7 = 449
trp_kingdom_6_lady_8 = 450
trp_kingdom_6_lady_9 = 451
trp_kingdom_6_lady_10 = 452
trp_kingdom_6_lady_11 = 453
trp_kingdom_6_lady_12 = 454
trp_kingdom_6_lady_13 = 455
trp_kingdom_6_lady_14 = 456
trp_kingdom_6_lady_15 = 457
trp_kingdom_6_lady_16 = 458
trp_kingdom_6_lady_17 = 459
trp_kingdom_6_lady_18 = 460
trp_kingdom_6_lady_19 = 461
trp_kingdom_6_lady_20 = 462
trp_heroes_end = 463
trp_town_1_seneschal = 464
trp_town_2_seneschal = 465
trp_town_3_seneschal = 466
trp_town_4_seneschal = 467
trp_town_5_seneschal = 468
trp_town_6_seneschal = 469
trp_town_7_seneschal = 470
trp_town_8_seneschal = 471
trp_town_9_seneschal = 472
trp_town_10_seneschal = 473
trp_town_11_seneschal = 474
trp_town_12_seneschal = 475
trp_town_13_seneschal = 476
trp_town_14_seneschal = 477
trp_town_15_seneschal = 478
trp_town_16_seneschal = 479
trp_town_17_seneschal = 480
trp_town_18_seneschal = 481
trp_town_19_seneschal = 482
trp_town_20_seneschal = 483
trp_town_21_seneschal = 484
trp_town_22_seneschal = 485
trp_castle_1_seneschal = 486
trp_castle_2_seneschal = 487
trp_castle_3_seneschal = 488
trp_castle_4_seneschal = 489
trp_castle_5_seneschal = 490
trp_castle_6_seneschal = 491
trp_castle_7_seneschal = 492
trp_castle_8_seneschal = 493
trp_castle_9_seneschal = 494
trp_castle_10_seneschal = 495
trp_castle_11_seneschal = 496
trp_castle_12_seneschal = 497
trp_castle_13_seneschal = 498
trp_castle_14_seneschal = 499
trp_castle_15_seneschal = 500
trp_castle_16_seneschal = 501
trp_castle_17_seneschal = 502
trp_castle_18_seneschal = 503
trp_castle_19_seneschal = 504
trp_castle_20_seneschal = 505
trp_castle_21_seneschal = 506
trp_castle_22_seneschal = 507
trp_castle_23_seneschal = 508
trp_castle_24_seneschal = 509
trp_castle_25_seneschal = 510
trp_castle_26_seneschal = 511
trp_castle_27_seneschal = 512
trp_castle_28_seneschal = 513
trp_castle_29_seneschal = 514
trp_castle_30_seneschal = 515
trp_castle_31_seneschal = 516
trp_castle_32_seneschal = 517
trp_castle_33_seneschal = 518
trp_castle_34_seneschal = 519
trp_castle_35_seneschal = 520
trp_castle_36_seneschal = 521
trp_castle_37_seneschal = 522
trp_castle_38_seneschal = 523
trp_castle_39_seneschal = 524
trp_castle_40_seneschal = 525
trp_castle_41_seneschal = 526
trp_castle_42_seneschal = 527
trp_castle_43_seneschal = 528
trp_castle_44_seneschal = 529
trp_castle_45_seneschal = 530
trp_castle_46_seneschal = 531
trp_castle_47_seneschal = 532
trp_castle_48_seneschal = 533
trp_town_1_arena_master = 534
trp_town_2_arena_master = 535
trp_town_3_arena_master = 536
trp_town_4_arena_master = 537
trp_town_5_arena_master = 538
trp_town_6_arena_master = 539
trp_town_7_arena_master = 540
trp_town_8_arena_master = 541
trp_town_9_arena_master = 542
trp_town_10_arena_master = 543
trp_town_11_arena_master = 544
trp_town_12_arena_master = 545
trp_town_13_arena_master = 546
trp_town_14_arena_master = 547
trp_town_15_arena_master = 548
trp_town_16_arena_master = 549
trp_town_17_arena_master = 550
trp_town_18_arena_master = 551
trp_town_19_arena_master = 552
trp_town_20_arena_master = 553
trp_town_21_arena_master = 554
trp_town_22_arena_master = 555
trp_town_1_armorer = 556
trp_town_2_armorer = 557
trp_town_3_armorer = 558
trp_town_4_armorer = 559
trp_town_5_armorer = 560
trp_town_6_armorer = 561
trp_town_7_armorer = 562
trp_town_8_armorer = 563
trp_town_9_armorer = 564
trp_town_10_armorer = 565
trp_town_11_armorer = 566
trp_town_12_armorer = 567
trp_town_13_armorer = 568
trp_town_14_armorer = 569
trp_town_15_armorer = 570
trp_town_16_armorer = 571
trp_town_17_armorer = 572
trp_town_18_armorer = 573
trp_town_19_armorer = 574
trp_town_20_armorer = 575
trp_town_21_armorer = 576
trp_town_22_armorer = 577
trp_town_1_weaponsmith = 578
trp_town_2_weaponsmith = 579
trp_town_3_weaponsmith = 580
trp_town_4_weaponsmith = 581
trp_town_5_weaponsmith = 582
trp_town_6_weaponsmith = 583
trp_town_7_weaponsmith = 584
trp_town_8_weaponsmith = 585
trp_town_9_weaponsmith = 586
trp_town_10_weaponsmith = 587
trp_town_11_weaponsmith = 588
trp_town_12_weaponsmith = 589
trp_town_13_weaponsmith = 590
trp_town_14_weaponsmith = 591
trp_town_15_weaponsmith = 592
trp_town_16_weaponsmith = 593
trp_town_17_weaponsmith = 594
trp_town_18_weaponsmith = 595
trp_town_19_weaponsmith = 596
trp_town_20_weaponsmith = 597
trp_town_21_weaponsmith = 598
trp_town_22_weaponsmith = 599
trp_town_1_tavernkeeper = 600
trp_town_2_tavernkeeper = 601
trp_town_3_tavernkeeper = 602
trp_town_4_tavernkeeper = 603
trp_town_5_tavernkeeper = 604
trp_town_6_tavernkeeper = 605
trp_town_7_tavernkeeper = 606
trp_town_8_tavernkeeper = 607
trp_town_9_tavernkeeper = 608
trp_town_10_tavernkeeper = 609
trp_town_11_tavernkeeper = 610
trp_town_12_tavernkeeper = 611
trp_town_13_tavernkeeper = 612
trp_town_14_tavernkeeper = 613
trp_town_15_tavernkeeper = 614
trp_town_16_tavernkeeper = 615
trp_town_17_tavernkeeper = 616
trp_town_18_tavernkeeper = 617
trp_town_19_tavernkeeper = 618
trp_town_20_tavernkeeper = 619
trp_town_21_tavernkeeper = 620
trp_town_22_tavernkeeper = 621
trp_town_1_merchant = 622
trp_town_2_merchant = 623
trp_town_3_merchant = 624
trp_town_4_merchant = 625
trp_town_5_merchant = 626
trp_town_6_merchant = 627
trp_town_7_merchant = 628
trp_town_8_merchant = 629
trp_town_9_merchant = 630
trp_town_10_merchant = 631
trp_town_11_merchant = 632
trp_town_12_merchant = 633
trp_town_13_merchant = 634
trp_town_14_merchant = 635
trp_town_15_merchant = 636
trp_town_16_merchant = 637
trp_town_17_merchant = 638
trp_town_18_merchant = 639
trp_town_19_merchant = 640
trp_town_20_merchant = 641
trp_town_21_merchant = 642
trp_town_22_merchant = 643
trp_salt_mine_merchant = 644
trp_town_1_horse_merchant = 645
trp_town_2_horse_merchant = 646
trp_town_3_horse_merchant = 647
trp_town_4_horse_merchant = 648
trp_town_5_horse_merchant = 649
trp_town_6_horse_merchant = 650
trp_town_7_horse_merchant = 651
trp_town_8_horse_merchant = 652
trp_town_9_horse_merchant = 653
trp_town_10_horse_merchant = 654
trp_town_11_horse_merchant = 655
trp_town_12_horse_merchant = 656
trp_town_13_horse_merchant = 657
trp_town_14_horse_merchant = 658
trp_town_15_horse_merchant = 659
trp_town_16_horse_merchant = 660
trp_town_17_horse_merchant = 661
trp_town_18_horse_merchant = 662
trp_town_19_horse_merchant = 663
trp_town_20_horse_merchant = 664
trp_town_21_horse_merchant = 665
trp_town_22_horse_merchant = 666
trp_town_1_mayor = 667
trp_town_2_mayor = 668
trp_town_3_mayor = 669
trp_town_4_mayor = 670
trp_town_5_mayor = 671
trp_town_6_mayor = 672
trp_town_7_mayor = 673
trp_town_8_mayor = 674
trp_town_9_mayor = 675
trp_town_10_mayor = 676
trp_town_11_mayor = 677
trp_town_12_mayor = 678
trp_town_13_mayor = 679
trp_town_14_mayor = 680
trp_town_15_mayor = 681
trp_town_16_mayor = 682
trp_town_17_mayor = 683
trp_town_18_mayor = 684
trp_town_19_mayor = 685
trp_town_20_mayor = 686
trp_town_21_mayor = 687
trp_town_22_mayor = 688
trp_village_1_elder = 689
trp_village_2_elder = 690
trp_village_3_elder = 691
trp_village_4_elder = 692
trp_village_5_elder = 693
trp_village_6_elder = 694
trp_village_7_elder = 695
trp_village_8_elder = 696
trp_village_9_elder = 697
trp_village_10_elder = 698
trp_village_11_elder = 699
trp_village_12_elder = 700
trp_village_13_elder = 701
trp_village_14_elder = 702
trp_village_15_elder = 703
trp_village_16_elder = 704
trp_village_17_elder = 705
trp_village_18_elder = 706
trp_village_19_elder = 707
trp_village_20_elder = 708
trp_village_21_elder = 709
trp_village_22_elder = 710
trp_village_23_elder = 711
trp_village_24_elder = 712
trp_village_25_elder = 713
trp_village_26_elder = 714
trp_village_27_elder = 715
trp_village_28_elder = 716
trp_village_29_elder = 717
trp_village_30_elder = 718
trp_village_31_elder = 719
trp_village_32_elder = 720
trp_village_33_elder = 721
trp_village_34_elder = 722
trp_village_35_elder = 723
trp_village_36_elder = 724
trp_village_37_elder = 725
trp_village_38_elder = 726
trp_village_39_elder = 727
trp_village_40_elder = 728
trp_village_41_elder = 729
trp_village_42_elder = 730
trp_village_43_elder = 731
trp_village_44_elder = 732
trp_village_45_elder = 733
trp_village_46_elder = 734
trp_village_47_elder = 735
trp_village_48_elder = 736
trp_village_49_elder = 737
trp_village_50_elder = 738
trp_village_51_elder = 739
trp_village_52_elder = 740
trp_village_53_elder = 741
trp_village_54_elder = 742
trp_village_55_elder = 743
trp_village_56_elder = 744
trp_village_57_elder = 745
trp_village_58_elder = 746
trp_village_59_elder = 747
trp_village_60_elder = 748
trp_village_61_elder = 749
trp_village_62_elder = 750
trp_village_63_elder = 751
trp_village_64_elder = 752
trp_village_65_elder = 753
trp_village_66_elder = 754
trp_village_67_elder = 755
trp_village_68_elder = 756
trp_village_69_elder = 757
trp_village_70_elder = 758
trp_village_71_elder = 759
trp_village_72_elder = 760
trp_village_73_elder = 761
trp_village_74_elder = 762
trp_village_75_elder = 763
trp_village_76_elder = 764
trp_village_77_elder = 765
trp_village_78_elder = 766
trp_village_79_elder = 767
trp_village_80_elder = 768
trp_village_81_elder = 769
trp_village_82_elder = 770
trp_village_83_elder = 771
trp_village_84_elder = 772
trp_village_85_elder = 773
trp_village_86_elder = 774
trp_village_87_elder = 775
trp_village_88_elder = 776
trp_village_89_elder = 777
trp_village_90_elder = 778
trp_village_91_elder = 779
trp_village_92_elder = 780
trp_village_93_elder = 781
trp_village_94_elder = 782
trp_village_95_elder = 783
trp_village_96_elder = 784
trp_village_97_elder = 785
trp_village_98_elder = 786
trp_village_99_elder = 787
trp_village_100_elder = 788
trp_village_101_elder = 789
trp_village_102_elder = 790
trp_village_103_elder = 791
trp_village_104_elder = 792
trp_village_105_elder = 793
trp_village_106_elder = 794
trp_village_107_elder = 795
trp_village_108_elder = 796
trp_village_109_elder = 797
trp_village_110_elder = 798
trp_merchants_end = 799
trp_town_1_master_craftsman = 800
trp_town_2_master_craftsman = 801
trp_town_3_master_craftsman = 802
trp_town_4_master_craftsman = 803
trp_town_5_master_craftsman = 804
trp_town_6_master_craftsman = 805
trp_town_7_master_craftsman = 806
trp_town_8_master_craftsman = 807
trp_town_9_master_craftsman = 808
trp_town_10_master_craftsman = 809
trp_town_11_master_craftsman = 810
trp_town_12_master_craftsman = 811
trp_town_13_master_craftsman = 812
trp_town_14_master_craftsman = 813
trp_town_15_master_craftsman = 814
trp_town_16_master_craftsman = 815
trp_town_17_master_craftsman = 816
trp_town_18_master_craftsman = 817
trp_town_19_master_craftsman = 818
trp_town_20_master_craftsman = 819
trp_town_21_master_craftsman = 820
trp_town_22_master_craftsman = 821
trp_zendar_chest = 822
trp_tutorial_chest_1 = 823
trp_tutorial_chest_2 = 824
trp_bonus_chest_1 = 825
trp_bonus_chest_2 = 826
trp_bonus_chest_3 = 827
trp_household_possessions = 828
trp_temp_array_a = 829
trp_temp_array_b = 830
trp_temp_array_c = 831
trp_stack_selection_amounts = 832
trp_stack_selection_ids = 833
trp_notification_menu_types = 834
trp_notification_menu_var1 = 835
trp_notification_menu_var2 = 836
trp_banner_background_color_array = 837
trp_multiplayer_data = 838
trp_local_merchant = 839
trp_tax_rebel = 840
trp_trainee_peasant = 841
trp_fugitive = 842
trp_belligerent_drunk = 843
trp_hired_assassin = 844
trp_fight_promoter = 845
trp_spy = 846
trp_spy_partner = 847
trp_nurse_for_lady = 848
trp_temporary_minister = 849
trp_quick_battle_6_player = 850
trp_swadian_crossbowman_multiplayer_ai = 851
trp_swadian_infantry_multiplayer_ai = 852
trp_swadian_man_at_arms_multiplayer_ai = 853
trp_vaegir_archer_multiplayer_ai = 854
trp_vaegir_spearman_multiplayer_ai = 855
trp_vaegir_horseman_multiplayer_ai = 856
trp_khergit_dismounted_lancer_multiplayer_ai = 857
trp_khergit_veteran_horse_archer_multiplayer_ai = 858
trp_khergit_lancer_multiplayer_ai = 859
trp_nord_veteran_multiplayer_ai = 860
trp_nord_scout_multiplayer_ai = 861
trp_nord_archer_multiplayer_ai = 862
trp_rhodok_veteran_crossbowman_multiplayer_ai = 863
trp_rhodok_veteran_spearman_multiplayer_ai = 864
trp_rhodok_scout_multiplayer_ai = 865
trp_sarranid_infantry_multiplayer_ai = 866
trp_sarranid_archer_multiplayer_ai = 867
trp_sarranid_horseman_multiplayer_ai = 868
trp_swadian_crossbowman_multiplayer = 869
trp_swadian_infantry_multiplayer = 870
trp_swadian_man_at_arms_multiplayer = 871
trp_vaegir_archer_multiplayer = 872
trp_vaegir_spearman_multiplayer = 873
trp_vaegir_horseman_multiplayer = 874
trp_khergit_veteran_horse_archer_multiplayer = 875
trp_khergit_lancer_multiplayer = 876
trp_nord_archer_multiplayer = 877
trp_nord_veteran_multiplayer = 878
trp_nord_scout_multiplayer = 879
trp_rhodok_veteran_crossbowman_multiplayer = 880
trp_rhodok_sergeant_multiplayer = 881
trp_rhodok_horseman_multiplayer = 882
trp_sarranid_archer_multiplayer = 883
trp_sarranid_footman_multiplayer = 884
trp_sarranid_mamluke_multiplayer = 885
trp_multiplayer_end = 886
trp_nurse = 887
trp_log_array_entry_type = 888
trp_log_array_entry_time = 889
trp_log_array_actor = 890
trp_log_array_center_object = 891
trp_log_array_center_object_lord = 892
trp_log_array_center_object_faction = 893
trp_log_array_troop_object = 894
trp_log_array_troop_object_faction = 895
trp_log_array_faction_object = 896
trp_quick_battle_troop_1 = 897
trp_quick_battle_troop_2 = 898
trp_quick_battle_troop_3 = 899
trp_quick_battle_troop_4 = 900
trp_quick_battle_troop_5 = 901
trp_quick_battle_troop_6 = 902
trp_quick_battle_troop_7 = 903
trp_quick_battle_troop_8 = 904
trp_quick_battle_troop_9 = 905
trp_quick_battle_troop_10 = 906
trp_quick_battle_troop_11 = 907
trp_quick_battle_troops_end = 908
trp_tutorial_fighter_1 = 909
trp_tutorial_fighter_2 = 910
trp_tutorial_fighter_3 = 911
trp_tutorial_fighter_4 = 912
trp_tutorial_archer_1 = 913
trp_tutorial_master_archer = 914
trp_tutorial_rider_1 = 915
trp_tutorial_rider_2 = 916
trp_tutorial_master_horseman = 917
trp_swadian_merchant = 918
trp_vaegir_merchant = 919
trp_khergit_merchant = 920
trp_nord_merchant = 921
trp_rhodok_merchant = 922
trp_sarranid_merchant = 923
trp_startup_merchants_end = 924
trp_sea_raider_leader = 925
trp_looter_leader = 926
trp_bandit_leaders_end = 927
trp_relative_of_merchant = 928
trp_relative_of_merchants_end = 929
trp_workshop_list = 930
trp_product_list = 931



## 用于编写一些ai方面的功能
from smart_module.module_system.header_dialogs import *
from smart_module.module_system.header_game_menus import *
from smart_module.module_system.header_operations import *
from smart_module.module_system.header_parties import *
from smart_module.module_system.header_terrain_types import rt_desert
from smart_module.module_system.module_constants import *
from smart_module.smart_core.utils.pythonUtils import mergeList

config = {
    ## 生长变化间隔时间(1月)
    "growInterval":7.2,
    ## 干旱变化间隔时间（1星期）
    "droughtInterval":1.68,
    ## 野草变化间隔时间（2星期）
    "weedInterval":3.36,
    ## 害虫变化间隔时间（2星期）
    "insectInterval":3.36,
    ## 工作变化间隔时间（2星期）
    "workInterval":0.24,
    ## 每过【workInterval】小时，增加健康值【workCount】
    "workCount":1,
    ## 每点【铁骨】技能增加工作进度点数
    "workSkill":skl_ironflesh,
    ## 每点技能增加的点数
    "workSkillCount":1,
    "plants":[
        {
            "plantName":"grain",
            "plantNameCn":"麦 子",
            ## 植物种类
            "plantType":"itm_grain",
            ## 种植费用
            "plantFee":100,
            ## 一块地种植时间（小时）
            "plantTime":2,
            ## 产量 20 * 30 - 100 = 500
            "plantQuantity":20,
        },
        {
            "plantName":"cabbage",
            "plantNameCn":"卷 心 菜",
            ## 植物种类
            "plantType":"itm_cabbages",
            ## 种植费用
            "plantFee":100,
            ## 一块地种植时间（小时）
            "plantTime":2,
            ## 产量 20 * 30 - 100 = 500
            "plantQuantity": 20,
        },
        {
            "plantName":"grape",
            "plantNameCn":"葡 萄",
            ## 植物种类
            "plantType":"itm_raw_grapes",
            ## 种植费用
            "plantFee":150,
            ## 一块地种植时间（小时）
            "plantTime":2,
            ## 产量 10 * 75 - 150 = 600
            "plantQuantity": 10,
        },
        {
            "plantName":"linen",
            "plantNameCn":"亚 麻",
            ## 植物种类
            "plantType":"itm_linen",
            ## 种植费用
            "plantFee":500,
            ## 一块地种植时间（小时）
            "plantTime":5,
            ## 产量 250 * 5 - 500 = 750
            "plantQuantity": 5,
        },
        {
            "plantName":"apple",
            "plantNameCn":"苹 果",
            ## 植物种类
            "plantType":"itm_apples",
            ## 种植费用
            "plantFee":120,
            ## 一块地种植时间（小时）
            "plantTime":5,
            ## 产量 44 * 15 - 120 = 540
            "plantQuantity": 15,
        },
        {
            "plantName":"olive",
            "plantNameCn":"橄 榄",
            ## 植物种类
            "plantType":"itm_raw_olives",
            ## 种植费用
            "plantFee":350,
            ## 一块地种植时间（小时）
            "plantTime":2,
            ## 产量 10 * 100 - 350 = 650
            "plantQuantity": 10,
        },
    ]
}

growInterval = config.get("growInterval")
droughtInterval = config.get("droughtInterval")
weedInterval = config.get("weedInterval")
insectInterval = config.get("insectInterval")

workCount = config.get("workCount")
workInterval = config.get("workInterval")
workSkill = config.get("workSkill")
workSkillCount = config.get("workSkillCount")

plants = config.get("plants")

## 测试状态
# growInterval = 0.1
# droughtInterval = 1
# weedInterval = 1
# insectInterval = 1
# workCount = 3

## $g_farmer_plant_type

## 管理任务模式
## $g_farmer_manage_mode
fmm_none = 1
fmm_planting = 2
fmm_watering = 3
fmm_weeding = 4
fmm_disinsection = 5
fmm_harvest = 6


## $g_farmer_land_planting
## $g_farmer_land_watering
## $g_farmer_land_weeding
## $g_farmer_land_disinsection
## $g_farmer_land_harvest

# $g_farmer_land_process
## 土地未种植
flp_none = 0
## 土地已经种植
flp_planted = 1
## 土地已经成熟
flp_plant_mature = 2
## 土地被毁坏了（洗劫以后）
flp_destroy = 3


# $g_farmer_work_enable
## 休息中
fwe_resting = 0
## 工作中
fwe_working = 1


def initFramerLive():
    plantOptions = []
    plantCnsList = []
    for plant in plants:
        plantName = plant.get("plantName")
        plantNameCn = plant.get("plantNameCn")
        plantType = plant.get("plantType")
        plantFee = plant.get("plantFee")
        plantTime = plant.get("plantTime")
        plantQuantity = plant.get("plantQuantity")

        optionName = "farmer_change_seed_" + plantName
        plantCnsList.append("mno_{}|{}（{}）".format(optionName,plantNameCn,plantFee))
        plantOptions.append((
            optionName,[
                (neg|eq,"$g_farmer_plant_type",plantType),
            ], plantName,[
                (eq, "$g_farmer_land_process", flp_none),
                (assign, "$g_farmer_plant_type", plantType),
                (assign, "$g_farmer_plant_fee", plantFee),
                (assign, "$g_farmer_plant_time", plantTime),
                (assign, "$g_farmer_plant_quantity", plantQuantity),
                (str_store_item_name,s0,plantType),
                (display_message, "@farmer change {s0}"),
                ## 切换到上一层菜单
                (jump_to_menu, "mnu_land_manage_options"),
            ]
        ))
    plantOptions.append(("farmer_change_seed_keep",[], "back",
                 [
                    (jump_to_menu, "mnu_land_manage_options"),
                 ]),)
    plantCnsList.append("mno_farmer_change_seed_keep|保 持 原 样")
    return {
        "plantOptions":plantOptions,
        "plantCnsList":plantCnsList,
    }

datas = initFramerLive()

plantOptions = datas.get("plantOptions")
plantCnsList = datas.get("plantCnsList")

red_color = 0xfffc0303
blue_color = 0xff0300f6
green_color = 0xff49b807

farmerLive={
    "name":"FarmerLive",
    "enable":True,
    "version": "v2.0.0",
    "desc":'''
        村民的生活
            1.定居
            2.管理土地
            3.种植农作物
            4.浇水
            5.除草
            6.除虫
            7.收割
            8.交税
    ''',
    "actions":[
        ("GameInitScript",[
            (assign,"$g_farmer_work_enable",fwe_resting),
            (assign,"$g_farmer_land_process",flp_none),
            (assign,"$g_farmer_manage_mode",fmm_none),

            (assign,"$g_farmer_plant_type",-1),
            (assign,"$g_farmer_plant_fee",-1),
            (assign,"$g_farmer_plant_time",-1),
            (assign,"$g_farmer_plant_quantity",-1),

            (assign,"$g_farmer_land_planting",0),
            (assign,"$g_farmer_land_watering",100),
            (assign,"$g_farmer_land_weeding",100),
            (assign,"$g_farmer_land_disinsection",100),
            (assign,"$g_farmer_land_harvest",0),

        ]),
        ("Append@scripts",[
            ## 开始工作
            ("start_work_at_field",[
                (assign, "$g_farmer_work_enable", fwe_working),
                (rest_for_hours_interactive, 100000, 5, 1),  # rest while not attackable
                (change_screen_return),
            ]),
            # 工作时调用 script_start_work_at_field
            ("update_field_work", [
                ##(assign,"$g_camp_mode", 1),
                ##(assign, "$g_infinite_camping", 0),
                ##(assign, "$g_player_icon_state", pis_ship),

                ## 是否在工作中
                (eq,"$g_farmer_work_enable",fwe_working),

                (store_skill_level, ":workSkill",workSkill,"trp_player"),
                # (assign,reg11,":workSkill"),
                # (display_message,"@workSkillCount {reg11}"),
                (store_mul,":workSkillCount",":workSkill",workSkillCount),
                (str_store_troop_name, s0, "trp_player"),
                ## 默认进度为1
                (assign,":baseCount",workCount),
                (store_add,":plusCount",":workSkillCount",":baseCount"),
                # (assign,reg10,":plusCount"),
                # (display_message,"@plusCount {reg10}"),
                (try_begin),
                    ## 未种植
                    (eq,"$g_farmer_land_process",flp_none),
                    ## 选择种植
                    (eq, "$g_farmer_manage_mode", fmm_planting),
                    (try_begin),
                        (lt,"$g_farmer_land_planting",100),
                        ## 进度增加1
                        (val_add, "$g_farmer_land_planting", ":plusCount"),
                        ## 限制范围0-100
                        (val_clamp,"$g_farmer_land_planting",0,101),
                        (assign,reg0,"$g_farmer_land_planting"),
                        (display_message,"@planting process {reg0}"),
                    (else_try),
                        ## 停止工作
                        (rest_for_hours, 0, 0, 0),
                        (assign,"$g_farmer_work_enable",fwe_resting),
                        (assign,"$g_farmer_land_process",flp_planted),
                        (assign,"$g_farmer_manage_mode",fmm_none),
                        ## 种子费用
                        (troop_remove_gold,"trp_player","$g_farmer_plant_fee"),
                        (display_message, "@{s0} planting is finished",blue_color),
                        ## 种植完成后初始化土地状态
                        (assign,"$g_farmer_land_watering",100),
                        (assign,"$g_farmer_land_weeding",100),
                        (assign,"$g_farmer_land_disinsection",100),
                    (try_end),
                (else_try),
                    ## 已种植
                    (eq,"$g_farmer_land_process",flp_planted),
                    (try_begin),
                        ## 浇水
                        (eq,"$g_farmer_manage_mode",fmm_watering),
                        (try_begin),
                            (lt,"$g_farmer_land_watering",100),
                            ## 进度增加1
                            (val_add, "$g_farmer_land_watering", ":plusCount"),
                            ## 限制范围0-100
                            (val_clamp, "$g_farmer_land_watering", 0, 101),
                            (assign,reg0,"$g_farmer_land_watering"),
                            (display_message,"@watering process {reg0}"),
                        (else_try),
                            (rest_for_hours, 0, 0, 0),
                            (assign,"$g_farmer_work_enable",fwe_resting),
                            (assign,"$g_farmer_manage_mode",fmm_none),
                            (display_message, "@{s0} watering is finished",blue_color),
                            ##(jump_to_menu, "mnu_land_manage_options"),
                        (try_end),
                    (else_try),
                        ## 除草
                        (eq,"$g_farmer_manage_mode",fmm_weeding),
                        (try_begin),
                            (lt,"$g_farmer_land_weeding",100),
                            ## 进度增加1
                            (val_add, "$g_farmer_land_weeding", ":plusCount"),
                            ## 限制范围0-100
                            (val_clamp, "$g_farmer_land_weeding", 0, 101),
                            (assign, reg0, "$g_farmer_land_weeding"),
                            (display_message, "@weeding process {reg0}"),
                        (else_try),
                            (rest_for_hours, 0, 0, 0),
                            (assign,"$g_farmer_work_enable",fwe_resting),
                            (assign,"$g_farmer_manage_mode",fmm_none),
                            (display_message, "@{s0} weeding is finished",blue_color),
                            ##(jump_to_menu, "mnu_land_manage_options"),
                        (try_end),
                    (else_try),
                        ## 除虫
                        (eq,"$g_farmer_manage_mode",fmm_disinsection),
                        (try_begin),
                            (lt,"$g_farmer_land_disinsection",100),
                            ## 进度增加1
                            (val_add, "$g_farmer_land_disinsection", ":plusCount"),
                            ## 限制范围0-100
                            (val_clamp, "$g_farmer_land_disinsection", 0, 101),
                            (assign, reg0, "$g_farmer_land_disinsection"),
                            (display_message, "@disinsection process {reg0}"),
                        (else_try),
                            (rest_for_hours, 0, 0, 0),
                            (assign,"$g_farmer_work_enable",fwe_resting),
                            (assign,"$g_farmer_manage_mode",fmm_none),
                            (display_message, "@{s0} disinsection is finished",blue_color),
                            ##(jump_to_menu, "mnu_land_manage_options"),
                        (try_end),
                    (else_try),
                        (display_message,"@plant has not planted"),
                    (try_end),
                (else_try),
                    ## 已成熟
                    (eq,"$g_farmer_land_process",flp_plant_mature),
                    ## 收割
                    (eq, "$g_farmer_manage_mode", fmm_harvest),
                    (try_begin),
                        (gt,"$g_farmer_land_harvest",0),
                        ## 进度增加1
                        (val_sub, "$g_farmer_land_harvest", ":plusCount"),
                        ## 限制范围0-100
                        (val_clamp, "$g_farmer_land_harvest", 0, 101),
                        (store_sub,":process",100,"$g_farmer_land_harvest"),
                        (assign, reg0, ":process"),
                        (display_message, "@harvest process {reg0}"),
                    (else_try),
                        ## 收获完成
                        (rest_for_hours, 0, 0, 0),
                        (assign,"$g_farmer_work_enable",fwe_resting),
                        (assign,"$g_farmer_land_process",flp_none),
                        (assign,"$g_farmer_manage_mode",fmm_none),
                        (display_message, "@{s0} harvest is finished",blue_color),
                        ## 种植的土地清空
                        (assign,"$g_farmer_land_planting",0),
                        ## 收获物品 ，全部存放到背包，如果超出就直接出售
                        (assign,":baseCount","$g_farmer_plant_quantity"),

                        (assign,":scale","$g_farmer_land_watering"),
                        (val_min,":scale","$g_farmer_land_weeding"),
                        (val_min,":scale","$g_farmer_land_disinsection"),

                        (store_mul,":quantity",":baseCount",":scale"),
                        (val_div,":quantity",100),
                        (troop_add_items,"trp_player","$g_farmer_plant_type",":quantity"),
                        ##(jump_to_menu, "mnu_land_manage_options"),
                    (try_end),
                (else_try),
                    ## 土地已销毁
                    (eq,"$g_farmer_land_process",flp_destroy),
                    (assign,":continue",0),
                (try_end),
            ]),
        ]),
        ("Append@simple_triggers",[
            ## 庄家生长（100：代表可以收获了）
            (growInterval,[
                (eq,"$g_farmer_land_process",flp_planted),
                (try_begin),
                    (lt,"$g_farmer_land_harvest",100),
                    (val_add,"$g_farmer_land_harvest",1),
                    (val_clamp,"$g_farmer_land_harvest",0,101),
                (else_try),
                    ## 农作物成熟了
                    (assign,"$g_farmer_land_process",flp_plant_mature),
                    (display_message,"@farmer plant is matured",green_color),
                (try_end),
            ]),
            ## 土地干涸（100：不缺水）
            (droughtInterval,[
                (eq,"$g_farmer_land_process",flp_planted),
                ## 没有人管理时
                (eq, "$g_farmer_manage_mode", fmm_none),
                (try_begin),
                    (gt,"$g_farmer_land_watering",0),
                    (val_sub,"$g_farmer_land_watering",1),
                    (val_clamp,"$g_farmer_land_watering",0,101),
                (else_try),
                    ## 农作物干涸死了
                    (assign,"$g_farmer_land_process",flp_none),
                    ## 清空农作物
                    (assign,"$g_farmer_land_planting",0),
                    ## 清空可收获农作物
                    (assign,"$g_farmer_land_harvest",0),
                    (display_message,"@farmer plant dry to death",red_color),
                (try_end),
            ]),
            ## 土地野草生长（100：没有杂草）
            (weedInterval,[
                ## 农作物已经种植
                (eq, "$g_farmer_land_process", flp_planted),
                ## 没有人管理时
                (eq, "$g_farmer_manage_mode", fmm_none),
                (try_begin),
                    (gt, "$g_farmer_land_weeding", 0),
                    (val_sub, "$g_farmer_land_weeding", 1),
                    (val_clamp, "$g_farmer_land_weeding", 0, 101),
                (else_try),
                    ## 农作物被野草淹没
                    (assign, "$g_farmer_land_process", flp_none),
                    ## 清空农作物
                    (assign, "$g_farmer_land_planting", 0),
                    ## 清空可收获农作物
                    (assign,"$g_farmer_land_harvest",0),
                    (display_message, "@farmer plant are submerged by weeds",red_color),
                (try_end),
            ]),
            ## 土地实害虫生长（100：没有虫子）
            (weedInterval,[
                (eq, "$g_farmer_land_process", flp_planted),
                ## 没有人管理时
                (eq, "$g_farmer_manage_mode", fmm_none),
                (try_begin),
                    (gt, "$g_farmer_land_disinsection", 0),
                    (val_sub, "$g_farmer_land_disinsection", 1),
                    (val_clamp, "$g_farmer_land_disinsection", 0, 101),
                (else_try),
                    ## 农作物被虫子吃完了
                    (assign, "$g_farmer_land_process", flp_none),
                    ## 清空农作物
                    (assign, "$g_farmer_land_planting", 0),
                    ## 清空可收获农作物
                    (assign,"$g_farmer_land_harvest",0),
                    (display_message, "@farmer plant were eaten up by insects",red_color),
                (try_end),
            ]),
            ## 玩家主动停止工作
            (0,[
                ## 如果在工作中
                (eq,"$g_farmer_work_enable",fwe_working),
                (map_free),
                (assign,"$g_farmer_work_enable",fwe_resting),
                (display_message,"@farmer stop work"),
             ]),
            ## 更新土地状态（野草，干旱，等等）
            (workInterval,[
                ## 如果在工作中
                (call_script,"script_update_field_work"),
             ])
        ]),
        ("AddDialogForVillage",[
            [anyone | plyr, "village_elder_talk", [], "I want to settle here.",
             "talk_elder_tobe_farmer_start", [
             ]],

            ## 同意加入
            [anyone, "talk_elder_tobe_farmer_start", [
                (troop_slot_eq,"trp_player",slot_troop_home,0),
            ], "Welcome to join us and do well.",
             "close_window", [
                ## 定居
                (troop_set_slot,"trp_player",slot_troop_home,"$current_town"),
                 (str_store_party_name,s0,"$current_town"),
                 (party_set_extra_text,"p_main_party","@{s0}'s farmer"),
             ]],

            ## 已经是别的村民
            [anyone, "talk_elder_tobe_farmer_start", [
                (neg|troop_slot_eq,"trp_player",slot_troop_home,0),
                (neg|troop_slot_eq,"trp_player",slot_troop_home,"$current_town"),
            ], "Sory,You are already a villager in another village!",
             "close_window", []],

            ## 已经是本村村民
            [anyone, "talk_elder_tobe_farmer_start", [
                (neg|troop_slot_eq,"trp_player",slot_troop_home,0),
                (troop_slot_eq,"trp_player",slot_troop_home,"$current_town"),
            ], "You are already a villager of the village.",
             "close_window", []],
        ]),
        ## 如果是村民，就拥有一块可以自己管理的土地
        ("Append@game_menus|village>5>village_manage",[
            ("land_manage",
             [
                 ## 玩家的家乡是当前村庄
                 (troop_slot_eq,"trp_player",slot_troop_home,"$current_town")
             ]
             , "Manage your land.",
             [
                 (jump_to_menu, "mnu_land_manage_options"),
             ]),
        ]),

        ## 不能打劫自己村庄
        ("Append@game_menus|village>5>village_hostile_action>1>#last",[
            (neg|troop_slot_eq,"trp_player",slot_troop_home,"$current_town"),
        ]),
        ## 能在村庄休息
        ("Append@game_menus|village>5>village_wait",[
            ("village_wait",
               [
                   (assign,":townLord","0"),
                   (try_begin),
                        (party_slot_eq, "$current_town", slot_center_has_manor, 1),
                        (party_slot_eq, "$current_town", slot_town_lord, "trp_player"),
                        (assign, ":townLord", "1"),
                   (try_end),
                    (this_or_next|troop_slot_eq,"trp_player",slot_troop_home,"$current_town"),
                   (eq,":townLord",1),

                ],
                 "Wait here for some time.",
                 [
                   (assign,"$auto_enter_town","$current_town"),
                   (assign, "$g_last_rest_center", "$current_town"),

                   (try_begin),
                     (party_is_active, "p_main_party"),
                     (party_get_current_terrain, ":cur_terrain", "p_main_party"),
                     (try_begin),
                       (eq, ":cur_terrain", rt_desert),
                       (unlock_achievement, ACHIEVEMENT_SARRANIDIAN_NIGHTS),
                     (try_end),
                   (try_end),
                   (rest_for_hours_interactive, 24 * 7, 5, 1), #rest while attackable
                   (change_screen_return),
          ]),

        ]),
        ("Append@game_menus",[
            ("land_manage_options", mnf_disable_all_keys,"Take good care of your land, and you will get rich rewards.","none",
             [],[
                ## 更换种子
                ("farmer_change_seed",[
                    ## (eq,"$g_farmer_plant_type",-1),
                ], "change seed",
                 [
                     (try_begin),
                        (eq,"$g_farmer_land_process",flp_none),
                        (assign, "$g_farmer_manage_mode", "mnu_village"),
                        (jump_to_menu, "mnu_farmer_change_seed_list"),
                    (else_try),
                        (eq, "$g_farmer_land_process", flp_planted),
                        (display_message,"@farmer is planted!"),
                    (else_try),
                        (eq, "$g_farmer_land_process", flp_plant_mature),
                        (display_message,"@farmer plant is matured"),
                    (else_try),
                        (eq, "$g_farmer_land_process", flp_destroy),
                        (display_message,"@farmer is destroyed!"),
                    (try_end),
                 ]),
                ## 种植
                ("farmer_plant_seed",[
                    (gt,"$g_farmer_plant_type",0),
                    (assign,reg0,"$g_farmer_land_planting"),
                ], "planting({reg0})",
                 [
                     (try_begin),
                        (eq,"$g_farmer_land_process",flp_none),
                        (assign, "$g_farmer_manage_mode",fmm_planting),
                        (call_script,"script_start_work_at_field"),
                     (else_try),
                        (display_message,"@framer land is not free"),
                     (try_end),
                 ]),
                ## 浇水
                ("farmer_watering",[
                    (eq,"$g_farmer_land_process",flp_planted),
                    (lt,"$g_farmer_land_watering",100),
                    (assign,reg1,"$g_farmer_land_watering"),
                ], "watering({reg1})",
                 [
                     (assign, "$g_farmer_manage_mode", fmm_watering),
                     (call_script, "script_start_work_at_field"),
                 ]),
                ## 除草
                ("farmer_weeding",[
                    (eq, "$g_farmer_land_process", flp_planted),
                    (lt, "$g_farmer_land_weeding", 100),
                    (assign, reg2, "$g_farmer_land_weeding"),
                ], "weeding({reg2})",
                 [
                     (assign, "$g_farmer_manage_mode", fmm_weeding),
                     (call_script, "script_start_work_at_field"),
                 ]),
                ## 除虫
                ("farmer_disinsection",[
                    (eq, "$g_farmer_land_process", flp_planted),
                    (lt, "$g_farmer_land_disinsection", 100),
                    (assign, reg3, "$g_farmer_land_disinsection"),
                ], "disinsection({reg3})",
                 [
                     (assign, "$g_farmer_manage_mode", fmm_disinsection),
                     (call_script, "script_start_work_at_field"),
                 ]),
                ## 收割
                ("farmer_harvest",[
                    (assign, reg4, "$g_farmer_land_harvest"),
                ], "harvest({reg4})",
                 [
                    (try_begin),
                        (eq,"$g_farmer_land_process",flp_plant_mature),
                        (assign, "$g_farmer_manage_mode",fmm_harvest),
                        (call_script, "script_start_work_at_field"),
                    (else_try),
                        (eq,"$g_farmer_land_process",flp_none),
                        (display_message,"@framer land is not planted"),
                    (else_try),
                        (display_message,"@plant is not mature"),
                    (try_end),
                 ]),
                ## 收割
                ("farmer_back",[], "back",
                 [
                    (jump_to_menu, "mnu_village"),
                 ]),
             ]),

            ## 切换种植类型
            ("farmer_change_seed_list", mnf_disable_all_keys,"Please choose the type you want to plant.","none",[],plantOptions),
        ]),
    ],
    ## 汉化
    "internationals":{
            "cns":{
                "game_menus":mergeList(plantCnsList,[
                    "mno_land_manage|管 理 你 的 土 地。",
                    "menu_land_manage_options|好 好 管 理 你 的 土 地 ， 你 会 得 到 丰 厚 的 回 报。",
                    "menu_farmer_change_seed_list|请 选 择 一 个 你 想 要 种 植 的 农 作 物 。",
                    "mno_farmer_change_seed|更 换 庄 家 种 类。",
                    "mno_farmer_plant_seed|开 始 种 植 （{reg0}）",
                    "mno_farmer_watering|开 始 浇 水 （{reg1}）",
                    "mno_farmer_weeding|开 始 除 草 （{reg2}）",
                    "mno_farmer_disinsection|开 始 除 虫 （{reg3}）",
                    "mno_farmer_harvest|开 始 收 获 （{reg4}）",
                    "mno_farmer_back|返 回",
                ]),
                "dialogs":[
                    "dlga_village_elder_talk:talk_elder_tobe_farmer_start|我 想 定 居 在 这 个 村 庄 。",
                    "dlga_talk_elder_tobe_farmer_start:close_window|欢 迎 加 入 我 们 ，好 好 干 。",
                    "dlga_talk_elder_tobe_farmer_start:close_window.1|对 不 起 ，你 已 经 加 入 别 的 村 庄 。",
                    "dlga_talk_elder_tobe_farmer_start:close_window.2|你 已 经 是 我 们 村 的 村 民 。",
                ],
                "quick_strings":[
                    "qstr_farmer_change_{s0}|更 改 家 作 物 类 型 为 {s0}",
                    "qstr_planting_process_{reg0}|种 植 进 度 ：{reg0}%",
                    "qstr_{s0}_planting_is_finished|{s0}：种 植 完 成 ！",
                    "qstr_watering_process_{reg0}|浇 水 进 度 ：{reg0}%",
                    "qstr_{s0}_watering_is_finished|{s0}：浇 水 完 成 ！",
                    "qstr_weeding_process_{reg0}|除 草 进 度 ：{reg0}%",
                    "qstr_{s0}_weeding_is_finished|{s0}：除 草 完 成 ！",
                    "qstr_disinsection_process_{reg0}|除 虫 进 度 ：{reg0}%",
                    "qstr_{s0}_disinsection_is_finished|{s0}：除 虫 完 成 ！",
                    "qstr_harvest_process_{reg0}|收 获 进 度 ：{reg0}%",
                    "qstr_{s0}_harvest_is_finished|{s0}：收 获 完 成 ！",
                    "qstr_farmer_stop_work|停 止 工 作 ！",
                    "qstr_farmer_is_planted_|农 作 物 已 经 种 植 了 ！",
                    "qstr_farmer_is_destroyed_|土 地 被 烧 毁 了 ！",
                    "qstr_farmer_plant_is_matured|家 作 物 成 熟 了 ！",
                    "qstr_plant_is_not_mature|家 作 物 还 没 有 成 熟 ！",

                    "qstr_framer_land_is_not_free|土 地 当 前 不 是 空 闲 的 ！",
                    "qstr_framer_land_is_not_planted|土 地 还 没 有 种 植 任 何 农 作 物 ！",

                    "qstr_farmer_plant_dry_to_death|农 作 物 已 经 干 旱 死 亡 ！",
                    "qstr_farmer_plant_are_submerged_by_weeds|农 作 物 已 经 杂 草 吞 噬 ！",
                    "qstr_farmer_plant_were_eaten_up_by_insects|农 作 物 已 经 害 虫 吃 完 ！",

                    "qstr_{s0}_s_farmer|{s0}的 村 民",
                ]
            }
    }
}


skill_str_begin = "str_skl_trade"
skill_str_end = "str_skl_end"

stringBaseScripts = {
    "name": "StringBaseScripts",
    "enable": True,
    "version": "v2.0.0",
    "desc": "与字符串相关的操作",
    "actions": [
        ("Append@strings", [
            ## 代码功能
            ("skl_trade", "trade"),
            ("skl_trade_desc", "trade_desc"),
            ("skl_leadership", "leadership"),
            ("skl_leadership_desc", "leadership_desc"),
            ("skl_prisoner_management", "prisoner_management"),
            ("skl_prisoner_management_desc", "prisoner_management_desc"),
            ("skl_reserved_1", "reserved_1"),
            ("skl_reserved_1_desc", "reserved_1_desc"),
            ("skl_reserved_2", "reserved_2"),
            ("skl_reserved_2_desc", "reserved_2_desc"),
            ("skl_reserved_3", "reserved_3"),
            ("skl_reserved_3_desc", "reserved_3_desc"),
            ("skl_reserved_4", "reserved_4"),
            ("skl_reserved_4_desc", "reserved_4_desc"),
            ("skl_persuasion", "persuasion"),
            ("skl_persuasion_desc", "persuasion_desc"),
            ("skl_engineer", "engineer"),
            ("skl_engineer_desc", "engineer_desc"),
            ("skl_first_aid", "first_aid"),
            ("skl_first_aid_desc", "first_aid_desc"),
            ("skl_surgery", "surgery"),
            ("skl_surgery_desc", "surgery_desc"),
            ("skl_wound_treatment", "wound_treatment"),
            ("skl_wound_treatment_desc", "wound_treatment_desc"),
            ("skl_inventory_management", "inventory_management"),
            ("skl_inventory_management_desc", "inventory_management_desc"),
            ("skl_spotting", "spotting"),
            ("skl_spotting_desc", "spotting_desc"),
            ("skl_pathfinding", "pathfinding"),
            ("skl_pathfinding_desc", "pathfinding_desc"),
            ("skl_tactics", "tactics"),
            ("skl_tactics_desc", "tactics_desc"),
            ("skl_tracking", "tracking"),
            ("skl_tracking_desc", "tracking_desc"),
            ("skl_trainer", "trainer"),
            ("skl_trainer_desc", "trainer_desc"),
            ("skl_reserved_5", "reserved_5"),
            ("skl_reserved_5_desc", "reserved_5_desc"),
            ("skl_reserved_6", "reserved_6"),
            ("skl_reserved_6_desc", "reserved_6_desc"),
            ("skl_reserved_7", "reserved_7"),
            ("skl_reserved_7_desc", "reserved_7_desc"),
            ("skl_reserved_8", "reserved_8"),
            ("skl_reserved_8_desc", "reserved_8_desc"),
            ("skl_looting", "looting"),
            ("skl_looting_desc", "looting_desc"),
            ("skl_horse_archery", "horse_archery"),
            ("skl_horse_archery_desc", "horse_archery_desc"),
            ("skl_riding", "riding"),
            ("skl_riding_desc", "riding_desc"),
            ("skl_athletics", "athletics"),
            ("skl_athletics_desc", "athletics_desc"),
            ("skl_shield", "shield"),
            ("skl_shield_desc", "shield_desc"),
            ("skl_weapon_master", "weapon_master"),
            ("skl_weapon_master_desc", "weapon_master_desc"),
            ("skl_reserved_9", "reserved_9"),
            ("skl_reserved_9_desc", "reserved_9_desc"),
            ("skl_reserved_10", "reserved_10"),
            ("skl_reserved_10_desc", "reserved_10_desc"),
            ("skl_reserved_11", "reserved_11"),
            ("skl_reserved_11_desc", "reserved_11_desc"),
            ("skl_reserved_12", "reserved_12"),
            ("skl_reserved_12_desc", "reserved_12_desc"),
            ("skl_reserved_13", "reserved_13"),
            ("skl_reserved_13_desc", "reserved_13_desc"),
            ("skl_power_draw", "power_draw"),
            ("skl_power_draw_desc", "power_draw_desc"),
            ("skl_power_throw", "power_throw"),
            ("skl_power_throw_desc", "power_throw_desc"),
            ("skl_power_strike", "power_strike"),
            ("skl_power_strike_desc", "power_strike_desc"),
            ("skl_ironflesh", "ironflesh"),
            ("skl_ironflesh_desc", "ironflesh_desc"),
            ("skl_reserved_14", "reserved_14"),
            ("skl_reserved_14_desc", "reserved_14_desc"),
            ("skl_reserved_15", "reserved_15"),
            ("skl_reserved_15_desc", "reserved_15_desc"),
            ("skl_reserved_16", "reserved_16"),
            ("skl_reserved_16_desc", "reserved_16_desc"),
            ("skl_reserved_17", "reserved_17"),
            ("skl_reserved_17_desc", "reserved_17_desc"),
            ("skl_reserved_18", "reserved_18"),
            ("skl_reserved_18_desc", "reserved_18_desc"),
            ("skl_end", "skill end")
        ]),
    ],
    "internationals": {
        "cns": {
            "game_strings": [
                "str_skl_trade|交 易 ",
                "str_skl_trade_desc|每 点 技 能 减 少 5%%的 交 易 损 失。（队 伍 技 能）",
                "str_skl_leadership|统 御 ",
                "str_skl_leadership_desc|每 点 增 加 部 队 命 令 上 限 5人， 提 高 部 队 士 气， 减 少 部 队 开 销 和 工 资 5%%。（领 袖 技 能）",
                "str_skl_prisoner_management|俘 虏 管 理 ",
                "str_skl_prisoner_management_desc|每 点 增 加 俘 虏 上 限 %d。（领 袖 技 能）",
                "str_skl_reserved_1|Reserved Skill 1",
                "str_skl_reserved_1_desc|This is a reserved skill.",
                "str_skl_reserved_2|Reserved Skill 2",
                "str_skl_reserved_2_desc|This is a reserved skill.",
                "str_skl_reserved_3|Reserved Skill 3",
                "str_skl_reserved_3_desc|This is a reserved skill.",
                "str_skl_reserved_4|Reserved Skill 4",
                "str_skl_reserved_4_desc|This is a reserved skill.",
                "str_skl_persuasion|说 服 力 ",
                "str_skl_persuasion_desc|此 技 能 帮 助 你， 使 你 的 观 点 更 容 易 让 人 接 受。 同 时 还 能 降 低 让 NPC 按 照 你 所 说 的 去 做 所 需 的 最 低 关 系 值。 （个 人 技 能）",
                "str_skl_engineer|工 程 学 ",
                "str_skl_engineer_desc|此 技 能 使 你 更 有 效 地 建 造 攻 城 器 械 和 加 快 封 地 建 设。（队 伍 技 能）",
                "str_skl_first_aid|急 救 ",
                "str_skl_first_aid_desc|每 点 返 还 给 英 雄 们 5%%在 上 次 战 斗 中 损 失 的 生 命 值。（队 伍 技 能）",
                "str_skl_surgery|手 术 ",
                "str_skl_surgery_desc|每 点 增 加 4%%的 机 会， 使 得 本 来 会 死 亡 的 队 伍 成 员 保 住 性 命。（队 伍 技 能）",
                "str_skl_wound_treatment|疗 伤 ",
                "str_skl_wound_treatment_desc|每 点 增 加 队 伍 生 命 值 回 复 速 度 20%%。（队 伍 技 能）",
                "str_skl_inventory_management|物 品 管 理 ",
                "str_skl_inventory_management_desc|每 点 增 加 物 品 栏 容 量 6格。（领 袖 技 能）",
                "str_skl_spotting|侦 察 ",
                "str_skl_spotting_desc|每 点 增 加 队 伍 侦 察 范 围 10%%。（队 伍 技 能）",
                "str_skl_pathfinding|向 导 ",
                "str_skl_pathfinding_desc|每 点 增 加 队 伍 地 图 移 动 速 度 3%%。（队 伍 技 能）",
                "str_skl_tactics|战 术 ",
                "str_skl_tactics_desc|每 2点 增 加 1点 战 斗 优 势。（敌 我 上 场 人 数）（队 伍 技 能）",
                "str_skl_tracking|跟 踪 ",
                "str_skl_tracking_desc|从 脚 印 和 其 它 印 迹 中 发 现 更 多 信 息。（队 伍 技 能）",
                "str_skl_trainer|教 练 ",
                "str_skl_trainer_desc|每 天 每 位 有 该 项 技 能 的 英 雄 会 增 加 经 验 值 给 其 它 级 别 较 低 的 队 伍 成 员。 传 授 的 经 验 值 依 次 为： {0, 4, 10, 16, 23, 30, 38, 46, 55, 65, 80}。（个 人 技 能）",
                "str_skl_reserved_5|Reserved Skill 5",
                "str_skl_reserved_5_desc|This is a reserved skill.",
                "str_skl_reserved_6|Reserved Skill 6",
                "str_skl_reserved_6_desc|This is a reserved skill.",
                "str_skl_reserved_7|Reserved Skill 7",
                "str_skl_reserved_7_desc|This is a reserved skill.",
                "str_skl_reserved_8|Reserved Skill 8",
                "str_skl_reserved_8_desc|This is a reserved skill.",
                "str_skl_looting|掠 夺 ",
                "str_skl_looting_desc|每 点 提 高 获 得 战 利 品 的 数 量 10%%。（队 伍 技 能）",
                "str_skl_horse_archery|骑 射 ",
                "str_skl_horse_archery_desc|减 少 在 马 背 上 射 箭 与 投 掷 的 精 度 和 伤 害 损 失。（个 人 技 能）",
                "str_skl_riding|骑 术 ",
                "str_skl_riding_desc|可 以 骑 更 好 的 马， 提 高 骑 行 速 度 和 操 纵。（个 人 技 能）",
                "str_skl_athletics|跑 动 ",
                "str_skl_athletics_desc|提 高 跑 动 速 度。（个 人 技 能）",
                "str_skl_shield|盾 防 ",
                "str_skl_shield_desc|减 少 盾 的 损 坏 程 度（每 点 减 少 8%%） 和 提 高 盾 的 格 挡 速 度， 以 及 覆 盖 范 围。（个 人 技 能）",
                "str_skl_weapon_master|武 器 掌 握 ",
                "str_skl_weapon_master_desc|让 武 器 熟 练 度 提 升 的 更 快 并 且 提 高 武 器 熟 练 度 的 上 限。 上 限 依 次 为： 60, 100, 140, 180, 220, 260, 300, 340, 380, 420。（个 人 技 能）",
                "str_skl_reserved_9|Reserved Skill 9",
                "str_skl_reserved_9_desc|This is a reserved skill.",
                "str_skl_reserved_10|Reserved Skill 10",
                "str_skl_reserved_10_desc|This is a reserved skill.",
                "str_skl_reserved_11|Reserved Skill 11",
                "str_skl_reserved_11_desc|This is a reserved skill.",
                "str_skl_reserved_12|Reserved Skill 12",
                "str_skl_reserved_12_desc|This is a reserved skill.",
                "str_skl_reserved_13|Reserved Skill 13",
                "str_skl_reserved_13_desc|This is a reserved skill.",
                "str_skl_power_draw|强 弓 ",
                "str_skl_power_draw_desc|角 色 可 以 使 用 更 强 力 的 弓。 每 点 增 加 弓 箭 伤 害 14%%， 最 高 有 效 等 级 不 超 过 弓 的 等 级 加 四。（个 人 技 能）",
                "str_skl_power_throw|强 掷 ",
                "str_skl_power_throw_desc|每 点 增 加 10%%的 投 掷 伤 害。（个 人 技 能）",
                "str_skl_power_strike|强 击 ",
                "str_skl_power_strike_desc|每 点 增 加 8%%的 格 斗 伤 害。（个 人 技 能）",
                "str_skl_ironflesh|铁 骨 ",
                "str_skl_ironflesh_desc|每 点 增 加 2点 生 命 值。（个 人 技 能）",
                "str_skl_reserved_14|Reserved Skill 14",
                "str_skl_reserved_14_desc|This is a reserved skill.",
                "str_skl_reserved_15|Reserved Skill 15",
                "str_skl_reserved_15_desc|This is a reserved skill.",
                "str_skl_reserved_16|Reserved Skill 16",
                "str_skl_reserved_16_desc|This is a reserved skill.",
                "str_skl_reserved_17|Reserved Skill 17",
                "str_skl_reserved_17_desc|This is a reserved skill.",
                "str_skl_reserved_18|Reserved Skill 18",
                "str_skl_reserved_18_desc|This is a reserved skill.",
                "str_skl_end|skill end.",
            ]
        }
    }
}



from smart_module.smart_core.base.meta_command.meta_command_processor import AppendProcessor, DeleteProcessor, \
    PrependProcessor, ReplaceProcessor

from smart_module.smart_core.utils.pythonUtils import mergeDict


################################################
###
### 处理器：语法糖，第三部分
### 系统内置语法糖
### 1.正常命令配置：必须提前配置，然后才能使用
###     key:命令名称
###     value:配置参数
###     主要属性：
###     1.【target】: 目标，命令作用在哪个数据集中，如：troops,strings,items,parties等等
###     2.【selector】：选择器，用于选择元素
###     3.【processor】：处理器，选择后，作相应的操作
###     4.【desc】：描述信息，描述命令的作用（可选），建议填写便于理解命令
###
################################################

## 语法糖（命令）合集

grammarSugarConfig = {}


## 通用命令
commonGrammarSugarConfig = {
    "Append":{
        "selector":"#last",
        "processor":AppendProcessor,
        "desc":"用于后置的命令"
    },
    "Prepend":{
        "selector":"#last",
        "processor":PrependProcessor,
        "desc":"用于前置的命令"
    },
    "Replace":{
        "selector":"#last",
        "processor":ReplaceProcessor,
        "desc":"用于替换的命令"
    },
    "Delete":{
        "selector":"#last",
        "processor":DeleteProcessor,
        "desc":"用于删除的命令"
    },
}

## 特殊场景命令
specialGrammarSugarConfig = {
    "GameInitScript":{
        "target":"scripts",
        "selector":"game_start>1>#last",
        "processor":AppendProcessor,
        "desc": "在游戏开始的脚本中增加自定义的语法"
    },
    "GameStartMenu":{
        "target":"game_menus",
        "selector":"start_game_0>5>#first",
        "processor":AppendProcessor,
        "desc": "在游戏开始的菜单中增加自定义的菜单"
    },
    "AppendCustomArrayTroop":{
        "target":"troops",
        "selector":"#last",
        "processor":AppendProcessor,
        "desc": "在游戏开始的脚本中增加自定义的语法"
    },
    "InitWalkers":{
        "target":"scripts",
        "selector":"init_town_walkers>1>#last",
        "processor":AppendProcessor,
        "desc": "访问村庄，城镇时，生成村民或市民，在此指令下，可以自定义自己的人物到村庄或城镇"
    },
    "AddCampOption":{
        "target":"game_menus",
        "selector":"camp>5>camp_action",
        "processor":AppendProcessor,
        "desc": "营地菜单添加新选项"
    },
    "AddDialogForVillage":{
        "target":"dialogs",
        "selector":"village_elder_talk&1:I_want_to_buy_some_supplies%^&3:village_elder_trade_begin&4",
        "processor":PrependProcessor,
        "desc": "为村长增加一个对话"
    }
}

## 合并语法糖集合
grammarSugarConfig = mergeDict(commonGrammarSugarConfig, specialGrammarSugarConfig)



script_game_start = 0
script_game_get_use_string = 1
script_game_quick_start = 2
script_get_army_size_from_slider_value = 3
script_spawn_quick_battle_army = 4
script_player_arrived = 5
script_game_set_multiplayer_mission_end = 6
script_game_enable_cheat_menu = 7
script_game_get_console_command = 8
script_game_event_party_encounter = 9
script_game_event_simulate_battle = 10
script_game_event_battle_end = 11
script_order_best_besieger_party_to_guard_center = 12
script_game_get_item_buy_price_factor = 13
script_game_get_item_sell_price_factor = 14
script_get_trade_penalty = 15
script_game_event_buy_item = 16
script_game_event_sell_item = 17
script_start_wedding_cutscene = 18
script_game_get_troop_wage = 19
script_game_get_total_wage = 20
script_game_get_join_cost = 21
script_game_get_upgrade_xp = 22
script_game_get_upgrade_cost = 23
script_game_get_prisoner_price = 24
script_game_check_prisoner_can_be_sold = 25
script_game_get_morale_of_troops_from_faction = 26
script_game_event_detect_party = 27
script_game_event_undetect_party = 28
script_game_get_statistics_line = 29
script_game_get_date_text = 30
script_game_get_money_text = 31
script_game_get_party_companion_limit = 32
script_game_reset_player_party_name = 33
script_game_get_troop_note = 34
script_game_get_center_note = 35
script_game_get_faction_note = 36
script_game_get_quest_note = 37
script_game_get_info_page_note = 38
script_game_get_scene_name = 39
script_game_get_mission_template_name = 40
script_add_kill_death_counts = 41
script_warn_player_about_auto_team_balance = 42
script_check_team_balance = 43
script_check_creating_ladder_dust_effect = 44
script_money_management_after_agent_death = 45
script_initialize_aristocracy = 46
script_initialize_trade_routes = 47
script_initialize_faction_troop_types = 48
script_initialize_item_info = 49
script_initialize_town_arena_info = 50
script_initialize_banner_info = 51
script_initialize_economic_information = 52
script_initialize_all_scene_prop_slots = 53
script_initialize_scene_prop_slots = 54
script_use_item = 55
script_determine_team_flags = 56
script_calculate_flag_move_time = 57
script_move_death_mode_flags_down = 58
script_move_flag = 59
script_move_headquarters_flags = 60
script_set_num_agents_around_flag = 61
script_change_flag_owner = 62
script_move_object_to_nearest_entry_point = 63
script_multiplayer_server_on_agent_spawn_common = 64
script_multiplayer_server_player_joined_common = 65
script_multiplayer_server_before_mission_start_common = 66
script_multiplayer_server_on_agent_killed_or_wounded_common = 67
script_multiplayer_close_gate_if_it_is_open = 68
script_multiplayer_move_moveable_objects_initial_positions = 69
script_move_belfries_to_their_first_entry_point = 70
script_team_set_score = 71
script_player_set_score = 72
script_player_set_kill_count = 73
script_player_set_death_count = 74
script_set_attached_scene_prop = 75
script_set_team_flag_situation = 76
script_start_death_mode = 77
script_calculate_new_death_waiting_time_at_death_mod = 78
script_calculate_number_of_targets_destroyed = 79
script_initialize_objects = 80
script_initialize_objects_clients = 81
script_show_multiplayer_message = 82
script_get_headquarters_scores = 83
script_draw_this_round = 84
script_check_achievement_last_man_standing = 85
script_find_most_suitable_bot_to_control = 86
script_game_receive_url_response = 87
script_game_get_cheat_mode = 88
script_game_receive_network_message = 89
script_cf_multiplayer_evaluate_poll = 90
script_multiplayer_accept_duel = 91
script_game_get_multiplayer_server_option_for_mission_template = 92
script_game_multiplayer_server_option_for_mission_template_to_string = 93
script_cf_multiplayer_team_is_available = 94
script_find_number_of_agents_constant = 95
script_game_multiplayer_event_duel_offered = 96
script_game_get_multiplayer_game_type_enum = 97
script_game_multiplayer_get_game_type_mission_template = 98
script_multiplayer_get_mission_template_game_type = 99
script_multiplayer_fill_available_factions_combo_button = 100
script_multiplayer_get_troop_class = 101
script_multiplayer_clear_player_selected_items = 102
script_multiplayer_init_player_slots = 103
script_multiplayer_initialize_belfry_wheel_rotations = 104
script_send_open_close_information_of_object = 105
script_multiplayer_send_initial_information = 106
script_multiplayer_remove_headquarters_flags = 107
script_multiplayer_remove_destroy_mod_targets = 108
script_multiplayer_init_mission_variables = 109
script_multiplayer_event_mission_end = 110
script_multiplayer_event_agent_killed_or_wounded = 111
script_multiplayer_get_item_value_for_troop = 112
script_multiplayer_get_previous_item_for_item_and_troop = 113
script_cf_multiplayer_is_item_default_for_troop = 114
script_multiplayer_calculate_cur_selected_items_cost = 115
script_multiplayer_set_item_available_for_troop = 116
script_multiplayer_send_item_selections = 117
script_multiplayer_set_default_item_selections_for_troop = 118
script_multiplayer_display_available_items_for_troop_and_item_classes = 119
script_multiplayer_fill_map_game_types = 120
script_multiplayer_count_players_bots = 121
script_multiplayer_find_player_leader_for_bot = 122
script_multiplayer_find_bot_troop_and_group_for_spawn = 123
script_multiplayer_change_leader_of_bot = 124
script_multiplayer_find_spawn_point = 125
script_multiplayer_find_spawn_point_2 = 126
script_multiplayer_buy_agent_equipment = 127
script_party_get_ideal_size = 128
script_game_get_party_prisoner_limit = 129
script_game_get_item_extra_text = 130
script_game_on_disembark = 131
script_game_context_menu_get_buttons = 132
script_game_event_context_menu_button_clicked = 133
script_game_get_skill_modifier_for_troop = 134
script_npc_get_troop_wage = 135
script_setup_talk_info = 136
script_setup_talk_info_companions = 137
script_update_party_creation_random_limits = 138
script_set_trade_route_between_centers = 139
script_average_trade_good_prices = 140
script_average_trade_good_prices_2 = 141
script_average_trade_good_productions = 142
script_normalize_trade_good_productions = 143
script_update_trade_good_prices = 144
script_update_trade_good_price_for_party = 145
script_center_get_production = 146
script_center_get_consumption = 147
script_get_enterprise_name = 148
script_do_merchant_town_trade = 149
script_party_calculate_regular_strength = 150
script_party_calculate_strength = 151
script_loot_player_items = 152
script_party_calculate_loot = 153
script_calculate_main_party_shares = 154
script_party_give_xp_and_gold = 155
script_setup_troop_meeting = 156
script_setup_party_meeting = 157
script_get_meeting_scene = 158
script_party_remove_all_companions = 159
script_party_remove_all_prisoners = 160
script_party_add_party_companions = 161
script_party_add_party_prisoners = 162
script_party_prisoners_add_party_companions = 163
script_party_prisoners_add_party_prisoners = 164
script_party_add_party = 165
script_party_copy = 166
script_clear_party_group = 167
script_party_add_wounded_members_as_prisoners = 168
script_get_nonempty_party_in_group = 169
script_collect_prisoners_from_empty_parties = 170
script_change_party_morale = 171
script_count_casualties_and_adjust_morale = 172
script_print_casualties_to_s0 = 173
script_write_fit_party_members_to_stack_selection = 174
script_remove_fit_party_member_from_stack_selection = 175
script_remove_random_fit_party_member_from_stack_selection = 176
script_add_routed_party = 177
script_cf_training_ground_sub_routine_1_for_melee_details = 178
script_training_ground_sub_routine_2_for_melee_details = 179
script_cf_training_ground_sub_routine_for_training_result = 180
script_print_troop_owned_centers_in_numbers_to_s0 = 181
script_get_random_melee_training_weapon = 182
script_start_training_at_training_ground = 183
script_party_count_fit_regulars = 184
script_party_count_fit_for_battle = 185
script_party_count_members_with_full_health = 186
script_get_stack_with_rank = 187
script_inflict_casualties_to_party = 188
script_move_members_with_ratio = 189
script_count_parties_of_faction_and_party_type = 190
script_faction_get_number_of_armies = 191
script_faction_recalculate_strength = 192
script_cf_select_random_town_with_faction = 193
script_cf_select_random_village_with_faction = 194
script_cf_select_random_walled_center_with_faction = 195
script_cf_select_random_walled_center_with_faction_and_owner_priority_no_siege = 196
script_cf_select_random_walled_center_with_faction_and_less_strength_priority = 197
script_cf_select_random_town_at_peace_with_faction = 198
script_cf_select_random_town_at_peace_with_faction_in_trade_route = 199
script_cf_select_most_profitable_town_at_peace_with_faction_in_trade_route = 200
script_shuffle_troop_slots = 201
script_get_quest = 202
script_get_dynamic_quest = 203
script_get_political_quest = 204
script_npc_find_quest_for_player_to_s11 = 205
script_cf_get_random_enemy_center_within_range = 206
script_cf_faction_get_random_enemy_faction = 207
script_cf_faction_get_random_friendly_faction = 208
script_cf_troop_get_random_enemy_troop_with_occupation = 209
script_cf_get_random_lord_in_a_center_with_faction = 210
script_cf_get_random_lord_except_king_with_faction = 211
script_cf_get_random_lord_from_another_faction_in_a_center = 212
script_get_closest_walled_center = 213
script_get_closest_center = 214
script_get_closest_center_of_faction = 215
script_get_closest_walled_center_of_faction = 216
script_let_nearby_parties_join_current_battle = 217
script_party_wound_all_members_aux = 218
script_party_wound_all_members = 219
script_calculate_battle_advantage = 220
script_cf_check_enemies_nearby = 221
script_get_heroes_attached_to_center_aux = 222
script_get_heroes_attached_to_center = 223
script_get_heroes_attached_to_center_as_prisoner_aux = 224
script_get_heroes_attached_to_center_as_prisoner = 225
script_give_center_to_faction = 226
script_give_center_to_faction_aux = 227
script_change_troop_faction = 228
script_troop_set_title_according_to_faction = 229
script_give_center_to_lord = 230
script_get_number_of_hero_centers = 231
script_cf_get_random_enemy_center = 232
script_find_travel_location = 233
script_get_relation_between_parties = 234
script_calculate_weekly_party_wage = 235
script_calculate_player_faction_wage = 236
script_calculate_hero_weekly_net_income_and_add_to_wealth = 237
script_cf_reinforce_party = 238
script_hire_men_to_kingdom_hero_party = 239
script_get_percentage_with_randomized_round = 240
script_create_cattle_herd = 241
script_buy_cattle_from_village = 242
script_kill_cattle_from_herd = 243
script_create_kingdom_hero_party = 244
script_create_kingdom_party_if_below_limit = 245
script_cf_create_kingdom_party = 246
script_get_troop_attached_party = 247
script_center_get_food_consumption = 248
script_center_get_food_store_limit = 249
script_refresh_village_merchant_inventory = 250
script_refresh_village_defenders = 251
script_village_set_state = 252
script_process_village_raids = 253
script_process_sieges = 254
script_lift_siege = 255
script_process_alarms = 256
script_allow_vassals_to_join_indoor_battle = 257
script_party_set_ai_state = 258
script_cf_party_under_player_suggestion = 259
script_troop_does_business_in_center = 260
script_process_kingdom_parties_ai = 261
script_process_hero_ai = 262
script_begin_assault_on_center = 263
script_select_faction_marshall = 264
script_get_center_faction_relation_including_player = 265
script_update_report_to_army_quest_note = 266
script_decide_faction_ai = 267
script_check_and_finish_active_army_quests_for_faction = 268
script_troop_get_player_relation = 269
script_change_troop_renown = 270
script_change_player_relation_with_troop = 271
script_change_player_relation_with_center = 272
script_change_player_relation_with_faction = 273
script_set_player_relation_with_faction = 274
script_change_player_relation_with_faction_ex = 275
script_cf_get_random_active_faction_except_player_faction_and_faction = 276
script_make_kingdom_hostile_to_player = 277
script_change_player_honor = 278
script_change_player_party_morale = 279
script_cf_player_has_item_without_modifier = 280
script_get_player_party_morale_values = 281
script_diplomacy_start_war_between_kingdoms = 282
script_diplomacy_party_attacks_neutral = 283
script_party_calculate_and_set_nearby_friend_enemy_follower_strengths = 284
script_init_ai_calculation = 285
script_recalculate_ais = 286
script_calculate_troop_ai = 287
script_diplomacy_start_peace_between_kingdoms = 288
script_event_kingdom_make_peace_with_kingdom = 289
script_randomly_start_war_peace_new = 290
script_exchange_prisoners_between_factions = 291
script_add_notification_menu = 292
script_finish_quest = 293
script_get_information_about_troops_position = 294
script_recruit_troop_as_companion = 295
script_setup_random_scene = 296
script_enter_dungeon = 297
script_enter_court = 298
script_setup_meet_lady = 299
script_find_high_ground_around_pos1 = 300
script_select_battle_tactic = 301
script_select_battle_tactic_aux = 302
script_battle_tactic_init = 303
script_battle_tactic_init_aux = 304
script_calculate_team_powers = 305
script_apply_effect_of_other_people_on_courage_scores = 306
script_apply_death_effect_on_courage_scores = 307
script_decide_run_away_or_not = 308
script_battle_tactic_apply = 309
script_battle_tactic_apply_aux = 310
script_team_get_class_percentages = 311
script_get_closest3_distance_of_enemies_at_pos1 = 312
script_team_get_average_position_of_enemies = 313
script_search_troop_prisoner_of_party = 314
script_change_debt_to_troop = 315
script_abort_quest = 316
script_cf_is_quest_troop = 317
script_check_friendly_kills = 318
script_simulate_retreat = 319
script_simulate_battle_with_agents_aux = 320
script_map_get_random_position_around_position_within_range = 321
script_get_number_of_unclaimed_centers_by_player = 322
script_cf_troop_check_troop_is_enemy = 323
script_troop_get_leaded_center_with_index = 324
script_cf_troop_get_random_leaded_walled_center_with_less_strength_priority = 325
script_cf_troop_get_random_leaded_town_or_village_except_center = 326
script_troop_write_owned_centers_to_s2 = 327
script_troop_write_family_relations_to_s1 = 328
script_troop_get_family_relation_to_troop = 329
script_collect_friendly_parties = 330
script_encounter_calculate_fit = 331
script_encounter_init_variables = 332
script_calculate_renown_value = 333
script_cf_get_first_agent_with_troop_id = 334
script_cf_team_get_average_position_of_agents_with_type_to_pos1 = 335
script_cf_turn_windmill_fans = 336
script_print_party_members = 337
script_round_value = 338
script_change_banners_and_chest = 339
script_remove_siege_objects = 340
script_describe_relation_to_s63 = 341
script_describe_center_relation_to_s3 = 342
script_center_ambiance_sounds = 343
script_center_set_walker_to_type = 344
script_cf_center_get_free_walker = 345
script_center_remove_walker_type_from_walkers = 346
script_init_town_walkers = 347
script_cf_enter_center_location_bandit_check = 348
script_init_town_agent = 349
script_init_town_walker_agents = 350
script_agent_get_town_walker_details = 351
script_town_walker_occupation_string_to_s14 = 352
script_tick_town_walkers = 353
script_set_town_walker_destination = 354
script_town_init_doors = 355
script_siege_init_ai_and_belfry = 356
script_cf_siege_move_belfry = 357
script_cf_siege_rotate_belfry_platform = 358
script_cf_siege_assign_men_to_belfry = 359
script_siege_move_archers_to_archer_positions = 360
script_store_movement_order_name_to_s1 = 361
script_store_riding_order_name_to_s1 = 362
script_store_weapon_usage_order_name_to_s1 = 363
script_team_give_order_from_order_panel = 364
script_update_order_panel = 365
script_update_agent_position_on_map = 366
script_convert_3d_pos_to_map_pos = 367
script_update_order_flags_on_map = 368
script_update_order_panel_checked_classes = 369
script_update_order_panel_statistics_and_map = 370
script_set_town_picture = 371
script_consume_food = 372
script_calculate_troop_score_for_center = 373
script_assign_lords_to_empty_centers = 374
script_create_village_farmer_party = 375
script_do_party_center_trade = 376
script_player_join_faction = 377
script_player_leave_faction = 378
script_deactivate_player_faction = 379
script_activate_player_faction = 380
script_agent_reassign_team = 381
script_start_quest = 382
script_conclude_quest = 383
script_succeed_quest = 384
script_fail_quest = 385
script_report_quest_troop_positions = 386
script_end_quest = 387
script_cancel_quest = 388
script_update_village_market_towns = 389
script_update_mercenary_units_of_towns = 390
script_update_volunteer_troops_in_village = 391
script_update_npc_volunteer_troops_in_village = 392
script_update_companion_candidates_in_taverns = 393
script_update_ransom_brokers = 394
script_update_tavern_travellers = 395
script_update_villages_infested_by_bandits = 396
script_update_booksellers = 397
script_update_tavern_minstrels = 398
script_update_other_taverngoers = 399
script_update_faction_notes = 400
script_update_faction_political_notes = 401
script_update_faction_traveler_notes = 402
script_update_troop_notes = 403
script_update_troop_location_notes = 404
script_update_troop_location_notes_prisoned = 405
script_update_troop_political_notes = 406
script_update_center_notes = 407
script_update_center_recon_notes = 408
script_update_all_notes = 409
script_agent_troop_get_banner_mesh = 410
script_shield_item_set_banner = 411
script_troop_agent_set_banner = 412
script_add_troop_to_cur_tableau = 413
script_add_troop_to_cur_tableau_for_character = 414
script_add_troop_to_cur_tableau_for_inventory = 415
script_add_troop_to_cur_tableau_for_profile = 416
script_add_troop_to_cur_tableau_for_retirement = 417
script_add_troop_to_cur_tableau_for_party = 418
script_get_prosperity_text_to_s50 = 419
script_spawn_bandits = 420
script_count_mission_casualties_from_agents = 421
script_get_max_skill_of_player_party = 422
script_upgrade_hero_party = 423
script_get_improvement_details = 424
script_cf_troop_agent_is_alive = 425
script_cf_village_recruit_volunteers_cond = 426
script_village_recruit_volunteers_recruit = 427
script_get_troop_item_amount = 428
script_get_name_from_dna_to_s50 = 429
script_change_center_prosperity = 430
script_get_center_ideal_prosperity = 431
script_good_price_affects_good_production = 432
script_get_poorest_village_of_faction = 433
script_troop_add_gold = 434
script_initialize_npcs = 435
script_objectionable_action = 436
script_post_battle_personality_clash_check = 437
script_event_player_defeated_enemy_party = 438
script_event_player_captured_as_prisoner = 439
script_npc_morale = 440
script_retire_companion = 441
script_reduce_companion_morale_for_clash = 442
script_calculate_ransom_amount_for_troop = 443
script_offer_ransom_amount_to_player_for_prisoners_in_party = 444
script_event_hero_taken_prisoner_by_player = 445
script_cf_check_hero_can_escape_from_player = 446
script_cf_party_remove_random_regular_troop = 447
script_place_player_banner_near_inventory = 448
script_place_player_banner_near_inventory_bms = 449
script_stay_captive_for_hours = 450
script_set_parties_around_player_ignore_player = 451
script_randomly_make_prisoner_heroes_escape_from_party = 452
script_fill_tournament_participants_troop = 453
script_get_num_tournament_participants = 454
script_get_random_tournament_participant = 455
script_add_tournament_participant = 456
script_get_random_tournament_team_amount_and_size = 457
script_get_troop_priority_point_for_tournament = 458
script_sort_tournament_participant_troops = 459
script_remove_tournament_participants_randomly = 460
script_end_tournament_fight = 461
script_get_win_amount_for_tournament_bet = 462
script_tournament_place_bet = 463
script_calculate_amount_of_cattle_can_be_stolen = 464
script_draw_banner_to_region = 465
script_get_troop_custom_banner_num_positionings = 466
script_get_custom_banner_charge_type_position_scale_color = 467
script_get_random_custom_banner = 468
script_get_custom_banner_color_from_index = 469
script_cf_check_color_visibility = 470
script_get_next_active_kingdom = 471
script_remove_cattles_if_herd_is_close_to_party = 472
script_get_rumor_to_s61 = 473
script_lord_comment_to_s43 = 474
script_add_log_entry = 475
script_get_relevant_comment_for_log_entry = 476
script_get_relevant_comment_to_s42 = 477
script_merchant_road_info_to_s42 = 478
script_get_manhunt_information_to_s15 = 479
script_rebellion_arguments = 480
script_get_culture_with_party_faction_for_music = 481
script_get_culture_with_faction_for_music = 482
script_music_set_situation_with_culture = 483
script_combat_music_set_situation_with_culture = 484
script_play_victorious_sound = 485
script_set_items_for_tournament = 486
script_custom_battle_end = 487
script_remove_troop_from_prison = 488
script_debug_variables = 489
script_troop_describes_troop_to_s15 = 490
script_troop_describes_quarrel_with_troop_to_s14 = 491
script_cf_test_lord_incompatibility_to_s17 = 492
script_troop_get_romantic_chemistry_with_troop = 493
script_cf_troop_get_romantic_attraction_to_troop = 494
script_cf_random_political_event = 495
script_evaluate_realm_stability = 496
script_battle_political_consequences = 497
script_faction_inflict_war_damage_on_faction = 498
script_calculate_troop_political_factors_for_liege = 499
script_cf_troop_can_intrigue = 500
script_troop_change_relation_with_troop = 501
script_troop_get_relation_with_troop = 502
script_appoint_faction_marshall = 503
script_center_get_item_consumption = 504
script_locate_player_minister = 505
script_lord_get_home_center = 506
script_get_kingdom_lady_social_determinants = 507
script_age_troop_one_year = 508
script_add_lady_items = 509
script_init_troop_age = 510
script_assign_troop_love_interests = 511
script_faction_conclude_feast = 512
script_lady_evaluate_troop_as_suitor = 513
script_courtship_event_troop_court_lady = 514
script_courtship_event_lady_break_relation_with_suitor = 515
script_courtship_event_bride_marry_groom = 516
script_npc_decision_checklist_party_ai = 517
script_npc_decision_checklist_troop_follow_or_not = 518
script_find_total_prosperity_score = 519
script_calculate_center_assailability_score = 520
script_find_center_to_defend = 521
script_npc_decision_checklist_peace_or_war = 522
script_npc_decision_checklist_male_guardian_assess_suitor = 523
script_npc_decision_checklist_marry_female_pc = 524
script_courtship_poem_reactions = 525
script_diplomacy_faction_get_diplomatic_status_with_faction = 526
script_faction_follows_controversial_policy = 527
script_internal_politics_rate_feast_to_s9 = 528
script_faction_get_adjective_to_s10 = 529
script_setup_tavern_attacker = 530
script_activate_tavern_attackers = 531
script_deactivate_tavern_attackers = 532
script_activate_town_guard = 533
script_cf_prisoner_offered_parole = 534
script_neutral_behavior_in_fight = 535
script_party_inflict_attrition = 536
script_add_rumor_string_to_troop_notes = 537
script_character_can_wed_character = 538
script_troop_change_career = 539
script_center_get_goods_availability = 540
script_lord_find_alternative_faction = 541
script_set_up_duel_with_troop = 542
script_test_player_for_career_and_marriage_incompatability = 543
script_deduct_casualties_from_garrison = 544
script_npc_decision_checklist_take_stand_on_issue = 545
script_npc_decision_checklist_evaluate_faction_strategy = 546
script_process_player_enterprise = 547
script_replace_scene_items_with_spawn_items_before_ms = 548
script_replace_scene_items_with_spawn_items_after_ms = 549
script_cf_is_melee_weapon_for_tutorial = 550
script_iterate_pointer_arrow = 551
script_find_center_to_attack_alt = 552
script_npc_decision_checklist_evaluate_enemy_center_for_attack = 553
script_npc_decision_checklist_faction_ai_alt = 554
script_faction_last_reconnoitered_center = 555
script_reduce_exact_number_to_estimate = 556
script_calculate_castle_prosperities_by_using_its_villages = 557
script_initialize_tavern_variables = 558
script_prepare_alley_to_fight = 559
script_prepare_town_to_fight = 560
script_change_player_right_to_rule = 561
script_indict_lord_for_treason = 562
script_give_center_to_faction_while_maintaining_lord = 563
script_check_concilio_calradi_achievement = 564
script_refresh_center_inventories = 565
script_refresh_center_armories = 566
script_refresh_center_weaponsmiths = 567
script_refresh_center_stables = 568
script_set_party_ai_go_to_center = 569
script_set_party_ai_patrol_center = 570
script_create_party = 571
script_party_add_members = 572
script_party_add_xp_and_upgrade = 573
script_party_change_ai_state = 574
script_get_prisoner_prices = 575
script_update_center_wealth = 576
script_party_handle_prisoners = 577
script_get_the_nearby_center = 578
script_get_all_center_arr_of_faction = 579
script_add_party_as_companions = 580
script_get_loard_all_centers = 581
script_get_center_of_lord = 582
script_get_party_max_skill = 583
script_get_num_of_enemy_state = 584
script_troop_leave_home = 585
script_troop_go_home = 586
script_set_random_name = 587
script_get_random_first_name = 588
script_get_random_second_name = 589
script_set_name_for_son = 590
script_display_troop_info = 591
script_set_age_in_range = 592
script_set_son_age = 593
script_troop_clear_items = 594
script_get_troop_all_items = 595
script_get_troop_all_wealth = 596
script_player_cosplay_anyone = 597
script_get_lord_strength = 598
script_update_lord_wealth = 599
script_get_troop_inventory_free_amount = 600
script_create_battle_for_player = 601
script_go_to_center = 602
script_update_prisperity_for_all_center = 603
script_ladies_go_out = 604
script_init_building_when_game_start = 605
script_get_patrol_center_strength = 606
script_get_patrol_ideal_size = 607
script_get_patrol_ideal_party_count = 608
script_get_patrol_center_need_create_num = 609
script_get_patrol_party_need_size = 610
script_update_patrol_partys_for_all = 611
script_update_all_patrol_party_faction = 612
script_init_village_base_basic_business_data = 613
script_get_current_center_workshop_admin = 614
script_start_work_at_wild = 615
script_start_work_at_field = 616
script_update_field_work = 617





import copy
import re

from smart_module.smart_core.base.exception import SmartModuleExceptin
from smart_module.smart_core.base.meta_command.meta_command_grammar_sugar_config import grammarSugarConfig
from smart_module.smart_core.base.meta_command.module_info_config import moduleDataMaping
from smart_module.smart_core.utils.pythonUtils import mergeDict


################################################
###
### 新功能-快捷指令解析器
### 快捷指令：
###     格式：命令名称@目标|选择器
###     命令必须是之前配置好的，快捷指令只是在已经有的命令上改变一些属性以便让指令适用性更广
###     优先级比较高：
###     1.@：替换target
###     2.|：替换selector
###
################################################

class QuickCommandParser:

    def __init__(self):
        ## 快捷指令格式
        self.quickCommandRegex = "^(\w+)(@(\w+))?(\|(.+))?$"

    def parseQuickCommand(self,quickCommand):
        '''
            解析快捷指令
        :param commandName:
        :return:
        '''
        matcher = re.match(self.quickCommandRegex, quickCommand)
        if not matcher:
            raise SmartModuleExceptin("快捷指令格式错误，请检查：" + quickCommand)
        return (matcher.group(1), matcher.group(3), matcher.group(5))

    def parseCommand(self,moduleInfoStr,quickCommand,customCommands,customDatas):
        '''
            解析命令数据
        :param quickCommand:
        :param datas:
        :return:
        '''
        commandInfo = self.parseQuickCommand(quickCommand)
        commandName = commandInfo[0]
        commandTarget = commandInfo[1]
        commandSelector = commandInfo[2]
        if commandName == None:
            raise SmartModuleExceptin("快捷指令必须提供名称，请检查：" + quickCommand)
        commands = mergeDict(grammarSugarConfig,customCommands if customCommands != None else [])
        buildCommand = commands.get(commandName)
        ## 深度复制，防止修改到内建指令
        command = copy.deepcopy(buildCommand)
        if command == None:
            raise SmartModuleExceptin("未知的快捷指令名称，请检查：" + commandName)
        ## 更新指令信息
        if commandTarget != None:
            command["target"] = commandTarget
        if commandSelector != None:
            command["selector"] = commandSelector
        target = command["target"]
        selector = command["selector"]
        processorType = command["processor"]
        dataConfig = moduleDataMaping.get(target)
        datas = dataConfig["datas"]
        level = dataConfig["level"]
        desc = command["desc"]
        return CommandInfo(commandName,target,selector,processorType,moduleInfoStr,customDatas,datas,level)


class CommandInfo:
    '''
        指令的相关数据
    '''
    def __init__(self,commandName,target,selector,processorType,moduleInfoStr,customDatas,datas,level):
        self.commandName = commandName
        self.target = target
        self.selector = selector
        self.processorType = processorType
        self.moduleInfoStr = moduleInfoStr
        self.customDatas = customDatas
        self.datas = datas
        self.level = level
    def __str__(self):
        return "{}:{}:{}:".format(self.commandName,self.target,self.level)


if __name__ == '__main__':
    quickCommandExecutor = QuickCommandParser()
    print(quickCommandExecutor.parseQuickCommand("Prepend"))
    print(quickCommandExecutor.parseQuickCommand("Prepend@strings"))
    print(quickCommandExecutor.parseQuickCommand("Prepend@strings|village_elder_talk&1:village_elder_trade_begin&4"))





from smart_module.module_system.header_skins import *
from smart_module.module_system.ID_particle_systems import *
####################################################################################################################
#  Each skin record contains the following fields:
#  1) Skin id: used for referencing skins.
#  2) Skin flags. Not used yet. Should be 0.
#  3) Body mesh.
#  4) Calf mesh (left one).
#  5) Hand mesh (left one).
#  6) Head mesh.
#  7) Face keys (list)
#  8) List of hair meshes.
#  9) List of beard meshes.
# 10) List of hair textures.
# 11) List of beard textures.
# 12) List of face textures.
# 13) List of voices.
# 14) Skeleton name
# 15) Scale (doesn't fully work yet)
# 16) Blood particles 1 (do not add this if you wish to use the default particles)
# 17) Blood particles 2 (do not add this if you wish to use the default particles)
# 17) Face key constraints (do not add this if you do not wish to use it)
####################################################################################################################

man_face_keys = [
(20,0, 0.7,-0.6, "Chin Size"),
(260,0, -0.6,1.4, "Chin Shape"),
(10,0,-0.5,0.9, "Chin Forward"),
(240,0,0.9,-0.8, "Jaw Width"),
(210,0,-0.5,1.0, "Jaw Position"),
(250,0,0.8,-1.0, "Mouth-Nose Distance"),
(200,0,-0.3,1.0, "Mouth Width"),
(50,0,-1.5,1.0, "Cheeks"),

(60,0,-0.4,1.35, "Nose Height"),
(70,0,-0.6,0.7, "Nose Width"),
(80,0,1.0,-0.1, "Nose Size"),
(270,0,-0.5,1.0, "Nose Shape"),
(90,0,-0.2,1.4, "Nose Bridge"),

(100,0,-0.3,1.5, "Cheek Bones"),
(150,0,-0.2,3.0, "Eye Width"),
(110,0,1.5,-0.9, "Eye to Eye Dist"),
(120,0,1.9,-1.0, "Eye Shape"),
(130,0,-0.5, 1.1, "Eye Depth"),
(140,0,1.0,-1.2, "Eyelids"),

(160,0,1.3,-0.2, "Eyebrow Position"),
(170,0,-0.1,1.9, "Eyebrow Height"),
(220,0,-0.1,0.9, "Eyebrow Depth"),
(180,0,-1.1,1.6, "Eyebrow Shape"),
(230,0,1.2,-0.7, "Temple Width"),

(30,0,-0.6,0.9, "Face Depth"),
(40,0,0.9,-0.6, "Face Ratio"),
(190,0,0.0,0.95, "Face Width"),

(280,0,0.0,1.0, "Post-Edit"),
]
# Face width-Jaw width Temple width
woman_face_keys = [
(230,0,0.8,-1.0, "Chin Size"), 
(220,0,-1.0,1.0, "Chin Shape"), 
(10,0,-1.2,1.0, "Chin Forward"),
(20,0, -0.6, 1.2, "Jaw Width"), 
(40,0,-0.7,1.0, "Jaw Position"),
(270,0,0.9,-0.9, "Mouth-Nose Distance"),
(30,0,-0.5,1.0, "Mouth Width"),
(50,0, -0.5,1.0, "Cheeks"),

(60,0,-0.5,1.0, "Nose Height"),
(70,0,-0.5,1.1, "Nose Width"),
(80,0,1.5,-0.3, "Nose Size"),
(240,0,-1.0,0.8, "Nose Shape"),
(90,0, 0.0,1.1, "Nose Bridge"),

(100,0,-0.5,1.5, "Cheek Bones"),
(150,0,-0.4,1.0, "Eye Width"),
(110,0,1.0,0.0, "Eye to Eye Dist"),
(120,0,-0.2,1.0, "Eye Shape"),
(130,0,-0.1,1.6, "Eye Depth"),
(140,0,-0.2,1.0, "Eyelids"),


(160,0,-0.2,1.2, "Eyebrow Position"),
(170,0,-0.2,0.7, "Eyebrow Height"),
(250,0,-0.4,0.9, "Eyebrow Depth"),
(180,0,-1.5,1.2, "Eyebrow Shape"),
(260,0,1.0,-0.7, "Temple Width"),

(200,0,-0.5,1.0, "Face Depth"),
(210,0,-0.5,0.9, "Face Ratio"),
(190,0,-0.4,0.8, "Face Width"),

(280,0,0.0,1.0, "Post-Edit"),
]
undead_face_keys = []


chin_size = 0
chin_shape = 1
chin_forward = 2
jaw_width = 3
jaw_position = 4
mouth_nose_distance = 5
mouth_width = 6
cheeks = 7
nose_height = 8
nose_width = 9
nose_size = 10
nose_shape = 11
nose_bridge = 12
cheek_bones = 13
eye_width = 14
eye_to_eye_dist = 15
eye_shape = 16
eye_depth = 17
eyelids = 18
eyebrow_position = 19
eyebrow_height = 20
eyebrow_depth = 21
eyebrow_shape = 22
temple_width = 23
face_depth = 24
face_ratio = 25
face_width = 26

comp_less_than = -1;
comp_greater_than = 1;

skins = [
  (
    "man", 0,
    "man_body", "man_calf_l", "m_handL",
    "male_head", man_face_keys,
    ["man_hair_s","man_hair_m","man_hair_n","man_hair_o", "man_hair_y10", "man_hair_y12","man_hair_p","man_hair_r","man_hair_q","man_hair_v","man_hair_t","man_hair_y6","man_hair_y3","man_hair_y7","man_hair_y9","man_hair_y11","man_hair_u","man_hair_y","man_hair_y2","man_hair_y4"], #man_hair_meshes ,"man_hair_y5","man_hair_y8",
    ["beard_e","beard_d","beard_k","beard_l","beard_i","beard_j","beard_z","beard_m","beard_n","beard_y","beard_p","beard_o",   "beard_v", "beard_f", "beard_b", "beard_c","beard_t","beard_u","beard_r","beard_s","beard_a","beard_h","beard_g",], #beard meshes ,"beard_q"
    ["hair_blonde", "hair_red", "hair_brunette", "hair_black", "hair_white"], #hair textures
    ["beard_blonde","beard_red","beard_brunette","beard_black","beard_white"], #beard_materials
    [("manface_young_2",0xffcbe0e0,["hair_blonde"],[0xffffffff, 0xffb04717, 0xff502a19]),
     ("manface_midage",0xffdfefe1,["hair_blonde"],[0xffffffff, 0xffb04717, 0xff632e18, 0xff502a19, 0xff19100c]),
     ("manface_young",0xffd0e0e0,["hair_blonde"],[0xff83301a, 0xff502a19, 0xff19100c, 0xff0c0d19]),     
#     ("manface_old",0xffd0d0d0,["hair_white","hair_brunette","hair_red","hair_blonde"],[0xffffcded, 0xffbbcded, 0xff99eebb]),
     ("manface_young_3",0xffdceded,["hair_blonde"],[0xff2f180e, 0xff171313, 0xff007080c]),
     ("manface_7",0xffc0c8c8,["hair_blonde"],[0xff171313, 0xff007080c]),
     ("manface_midage_2",0xfde4c8d8,["hair_blonde"],[0xff502a19, 0xff19100c, 0xff0c0d19]),
     ("manface_rugged",0xffb0aab5,["hair_blonde"],[0xff171313, 0xff007080c]),
#     ("manface_young_4",0xffe0e8e8,["hair_blonde"],[0xff2f180e, 0xff171313, 0xff007080c]),
     ("manface_african",0xff807c8a,["hair_blonde"],[0xff120808, 0xff007080c]),     
#     ("manface_old_2",0xffd5d5c5,["hair_white"],[0xffffcded, 0xffbbcded, 0xff99eebb]),
     ], #man_face_textures,
    [(voice_die,"snd_man_die"),(voice_hit,"snd_man_hit"),(voice_grunt,"snd_man_grunt"),(voice_grunt_long,"snd_man_grunt_long"),(voice_yell,"snd_man_yell"),(voice_stun,"snd_man_stun"),(voice_victory,"snd_man_victory")], #voice sounds
    "skel_human", 1.0,
    psys_game_blood,psys_game_blood_2,
    [[1.7, comp_greater_than, (1.0,face_width), (1.0,temple_width)], #constraints: ex: 1.7 > (face_width + temple_width)
     [0.3, comp_less_than, (1.0,face_width), (1.0,temple_width)],
     [1.7, comp_greater_than, (1.0,face_width), (1.0,face_depth)],
     [0.3, comp_less_than, (1.0,eyebrow_height), (1.0,eyebrow_position)],
     [1.7, comp_greater_than, (1.0,eyebrow_height), (1.0,eyebrow_position)],
     [-0.7, comp_less_than, (1.0,nose_size), (-1.0,nose_shape)],
     [0.7, comp_greater_than, (1.0,nose_size), (-1.0,nose_shape)],
     [2.7, comp_greater_than, (1.0,chin_size), (1.0,mouth_nose_distance), (1.0,nose_height), (-1.0,face_width)],
     ]
  ),
  
  (
    "woman", skf_use_morph_key_10,
    "woman_body",  "woman_calf_l", "f_handL",
    "female_head", woman_face_keys,
    ["woman_hair_p","woman_hair_n","woman_hair_o","woman_hair_q","woman_hair_r","woman_hair_t","woman_hair_s"], #woman_hair_meshes
#    ["woman_hair_a","woman_hair_b","woman_hair_c","woman_hair_d","woman_hair_e","woman_hair_f","woman_hair_g"], #woman_hair_meshes
    [],
    ["hair_blonde", "hair_red", "hair_brunette", "hair_black", "hair_white"], #hair textures
    [],
    [("womanface_young",0xffe3e8ef,["hair_blonde"],[0xffffffff, 0xffb04717, 0xff502a19, 0xff19100c]),
     ("womanface_b",0xffdfdfdf,["hair_blonde"],[0xffa5481f, 0xff502a19, 0xff19100c, 0xff0c0d19]),
     ("womanface_a",0xffe8dfe5,["hair_blonde"],[0xff502a19, 0xff19100c, 0xff0c0d19]),
     ("womanface_brown",0xffaf9f7e,["hair_blonde"],[0xff19100c, 0xff0c0d19, 0xff007080c]),
     ("womanface_african",0xff808080,["hair_blonde"],[0xff120808, 0xff007080c]),
#     ("womanface_midage",0xffe5eaf0,["hair_black","hair_brunette","hair_red","hair_white"],[0xffffcded, 0xffbbcded, 0xff99eebb]),
     ],#woman_face_textures
    [(voice_die,"snd_woman_die"),(voice_hit,"snd_woman_hit"),(voice_yell,"snd_woman_yell")], #voice sounds
    "skel_human", 1.0,
    psys_game_blood,psys_game_blood_2,
  ),
  
##  (
##    "undead", 0,
##    "undead_body", "undead_calf_l", "undead_handL",
##    "undead_head", undead_face_keys,
##    [],
##    [],
##    [],
##    [],
##    [("undeadface_a",0xffffffff,[]),
##     ("undeadface_b",0xffcaffc0,[]),
##     ], #undead_face_textures
##    [], #voice sounds
##    "skel_human", 1.0,
##  ),
]




arf_blend_in_0               = 0x00000001
arf_blend_in_1               = 0x00000002
arf_blend_in_2               = 0x00000003
arf_blend_in_3               = 0x00000004
arf_blend_in_4               = 0x00000005
arf_blend_in_5               = 0x00000006
arf_blend_in_6               = 0x00000007
arf_blend_in_7               = 0x00000008
arf_blend_in_8               = 0x00000009
arf_blend_in_9               = 0x0000000a
arf_blend_in_10              = 0x0000000b
arf_blend_in_11              = 0x0000000c
arf_blend_in_12              = 0x0000000d
arf_blend_in_13              = 0x0000000e
arf_blend_in_14              = 0x0000000f
arf_blend_in_15              = 0x00000010
arf_blend_in_16              = 0x00000011
arf_blend_in_17              = 0x00000012
arf_blend_in_18              = 0x00000013
arf_blend_in_19              = 0x00000014
arf_blend_in_20              = 0x00000015
arf_blend_in_21              = 0x00000016
arf_blend_in_22              = 0x00000017
arf_blend_in_23              = 0x00000018
arf_blend_in_24              = 0x00000019
arf_blend_in_25              = 0x0000001a
arf_blend_in_26              = 0x0000001b
arf_blend_in_27              = 0x0000001c
arf_blend_in_28              = 0x0000001d
arf_blend_in_29              = 0x0000001e
arf_blend_in_30              = 0x0000001f
arf_blend_in_31              = 0x00000020
arf_blend_in_32              = 0x00000021
arf_blend_in_48              = 0x00000031
arf_blend_in_64              = 0x00000041
arf_blend_in_128             = 0x00000081
arf_blend_in_254             = 0x000000ff

arf_make_walk_sound          = 0x00000100
arf_make_custom_sound        = 0x00000200

##arf_start_pos_0               = 0x00000100
##arf_end_pos_0                 = 0x00000200
##arf_start_pos_0_25            = 0x00000400
##arf_end_pos_0_25              = 0x00000800
##arf_start_pos_0_5             = 0x00001000
##arf_end_pos_0_5               = 0x00002000
##arf_start_pos_0_75            = 0x00004000
##arf_end_pos_0_75              = 0x00008000

##arf_loop_pos_0     = arf_start_pos_0    | arf_end_pos_0
##arf_loop_pos_0_25  = arf_start_pos_0_25 | arf_end_pos_0_25
##arf_loop_pos_0_5   = arf_start_pos_0_5  | arf_end_pos_0_5
##arf_loop_pos_0_75  = arf_start_pos_0_75 | arf_end_pos_0_75

##arf_phase_even               = 0x00010000
##arf_phase_odd                = 0x00030000
##arf_phase_inverse_even       = 0x00050000
##arf_phase_inverse_odd        = 0x00070000
arf_two_handed_blade         = 0x01000000
arf_lancer                   = 0x02000000
arf_stick_item_to_left_hand  = 0x04000000
arf_cyclic                   = 0x10000000

arf_use_walk_progress        = 0x20000000
arf_use_stand_progress       = 0x40000000
arf_use_inv_walk_progress    = 0x80000000

##arf_walk = arf_phase_even | arf_cyclic

#-----------------------------------------

amf_priority_mask                          = 0x00000fff

amf_rider_rot_bow                          = 0x00001000
amf_rider_rot_throw                        = 0x00002000
amf_rider_rot_crossbow                     = 0x00003000
amf_rider_rot_pistol                       = 0x00004000
amf_rider_rot_overswing                    = 0x00005000
amf_rider_rot_thrust                       = 0x00006000
amf_rider_rot_swing_right                  = 0x00007000
amf_rider_rot_swing_left                   = 0x00008000
amf_rider_rot_couched_lance                = 0x00009000
amf_rider_rot_shield                       = 0x0000a000
amf_rider_rot_defend                       = 0x0000b000

amf_start_instantly                        = 0x00010000
amf_use_cycle_period                       = 0x00100000
amf_use_weapon_speed                       = 0x00200000
amf_use_defend_speed                       = 0x00400000
amf_accurate_body                          = 0x00800000
amf_client_prediction                      = 0x01000000
amf_play                                   = 0x02000000
amf_keep	                                  = 0x04000000
amf_restart                                = 0x08000000 # restart animation even if it is the current animation
amf_hide_weapon                            = 0x10000000
amf_client_owner_prediction                = 0x20000000
amf_use_inertia                            = 0x40000000
amf_continue_to_next                       = 0x80000000

#-----------------------------------------

acf_synch_with_horse         = 0x00000001
acf_align_with_ground        = 0x00000002
acf_enforce_lowerbody        = 0x00000100
acf_enforce_rightside        = 0x00000200
acf_enforce_all              = 0x00000400
acf_parallels_for_look_slope = 0x00001000
acf_lock_camera              = 0x00002000
acf_displace_position        = 0x00004000
acf_ignore_slope             = 0x00008000
acf_thrust                   = 0x00010000
acf_right_cut                = 0x00020000
acf_left_cut                 = 0x00040000
acf_overswing                = 0x00080000
acf_rot_vertical_mask        = 0x00300000
acf_rot_vertical_bow         = 0x00100000
acf_rot_vertical_sword       = 0x00200000
acf_anim_length_mask         = 0xff000000

acf_anim_length_bits         = 24
def acf_anim_length(x):
  return (x << acf_anim_length_bits) & acf_anim_length_mask
#------------------------------------------



#### Do not edit these lines

def get_byte(f):
  if f == 0.0:
    return 0
  i = int(f * 255.0)
  if (i< 1):
    i=1
  elif (i > 255):
    i = 255
  return i

def pack2f(a,b):
  ai = get_byte(a)
  bi = get_byte(b)
  return ((bi << 8) | ai)

def pack4f(a,b,c,d):
  ai = get_byte(a)
  bi = get_byte(b)
  ci = get_byte(c)
  di = get_byte(d)
  return ((di << 24) | (ci << 16) | (bi << 8) | ai)



from smart_module.module_system.module_info import *
from smart_module.module_system.module_music import *

def save_python_header(src_dir):
  ofile = open(src_dir + "ID_music.py","w",encoding="utf-8")
  for i_track in range(len(tracks)):
    ofile.write("track_%s = %d\n"%(tracks[i_track][0],i_track))
  ofile.write("\n\n")
  ofile.close()

def save_tracks(export_dir):
  file = open(export_dir + "music.txt","w",encoding="utf-8")
  file.write("%d\n"%len(tracks))
  for track in tracks:
    file.write("%s %d %d\n"%(track[1], track[2], (track[2] | track[3])))
  file.close()


def processMusic(context):
  print( "Exporting tracks...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  save_python_header(src_dir)
  save_tracks(export_dir)



qst_deliver_message = 0
qst_deliver_message_to_enemy_lord = 1
qst_raise_troops = 2
qst_escort_lady = 3
qst_deal_with_bandits_at_lords_village = 4
qst_collect_taxes = 5
qst_hunt_down_fugitive = 6
qst_kill_local_merchant = 7
qst_bring_back_runaway_serfs = 8
qst_follow_spy = 9
qst_capture_enemy_hero = 10
qst_lend_companion = 11
qst_collect_debt = 12
qst_incriminate_loyal_commander = 13
qst_meet_spy_in_enemy_town = 14
qst_capture_prisoners = 15
qst_lend_surgeon = 16
qst_follow_army = 17
qst_report_to_army = 18
qst_deliver_cattle_to_army = 19
qst_join_siege_with_army = 20
qst_screen_army = 21
qst_scout_waypoints = 22
qst_rescue_lord_by_replace = 23
qst_deliver_message_to_prisoner_lord = 24
qst_duel_for_lady = 25
qst_duel_courtship_rival = 26
qst_duel_avenge_insult = 27
qst_move_cattle_herd = 28
qst_escort_merchant_caravan = 29
qst_deliver_wine = 30
qst_troublesome_bandits = 31
qst_kidnapped_girl = 32
qst_persuade_lords_to_make_peace = 33
qst_deal_with_looters = 34
qst_deal_with_night_bandits = 35
qst_deliver_grain = 36
qst_deliver_cattle = 37
qst_train_peasants_against_bandits = 38
qst_eliminate_bandits_infesting_village = 39
qst_visit_lady = 40
qst_formal_marriage_proposal = 41
qst_obtain_liege_blessing = 42
qst_wed_betrothed = 43
qst_wed_betrothed_female = 44
qst_join_faction = 45
qst_rebel_against_kingdom = 46
qst_consult_with_minister = 47
qst_organize_feast = 48
qst_resolve_dispute = 49
qst_offer_gift = 50
qst_denounce_lord = 51
qst_intrigue_against_lord = 52
qst_track_down_bandits = 53
qst_track_down_provocateurs = 54
qst_retaliate_for_border_incident = 55
qst_raid_caravan_to_start_war = 56
qst_cause_provocation = 57
qst_rescue_prisoner = 58
qst_destroy_bandit_lair = 59
qst_blank_quest_2 = 60
qst_blank_quest_3 = 61
qst_blank_quest_4 = 62
qst_blank_quest_5 = 63
qst_blank_quest_6 = 64
qst_blank_quest_7 = 65
qst_blank_quest_8 = 66
qst_blank_quest_9 = 67
qst_blank_quest_10 = 68
qst_blank_quest_11 = 69
qst_blank_quest_12 = 70
qst_blank_quest_13 = 71
qst_blank_quest_14 = 72
qst_blank_quest_15 = 73
qst_blank_quest_16 = 74
qst_blank_quest_17 = 75
qst_blank_quest_18 = 76
qst_blank_quest_19 = 77
qst_blank_quest_20 = 78
qst_blank_quest_21 = 79
qst_blank_quest_22 = 80
qst_blank_quest_23 = 81
qst_blank_quest_24 = 82
qst_blank_quest_25 = 83
qst_blank_quest_26 = 84
qst_blank_quest_27 = 85
qst_collect_men = 86
qst_learn_where_merchant_brother_is = 87
qst_save_relative_of_merchant = 88
qst_save_town_from_bandits = 89
qst_quests_end = 90
qsttag_deliver_message = 504403158265495552
qsttag_deliver_message_to_enemy_lord = 504403158265495553
qsttag_raise_troops = 504403158265495554
qsttag_escort_lady = 504403158265495555
qsttag_deal_with_bandits_at_lords_village = 504403158265495556
qsttag_collect_taxes = 504403158265495557
qsttag_hunt_down_fugitive = 504403158265495558
qsttag_kill_local_merchant = 504403158265495559
qsttag_bring_back_runaway_serfs = 504403158265495560
qsttag_follow_spy = 504403158265495561
qsttag_capture_enemy_hero = 504403158265495562
qsttag_lend_companion = 504403158265495563
qsttag_collect_debt = 504403158265495564
qsttag_incriminate_loyal_commander = 504403158265495565
qsttag_meet_spy_in_enemy_town = 504403158265495566
qsttag_capture_prisoners = 504403158265495567
qsttag_lend_surgeon = 504403158265495568
qsttag_follow_army = 504403158265495569
qsttag_report_to_army = 504403158265495570
qsttag_deliver_cattle_to_army = 504403158265495571
qsttag_join_siege_with_army = 504403158265495572
qsttag_screen_army = 504403158265495573
qsttag_scout_waypoints = 504403158265495574
qsttag_rescue_lord_by_replace = 504403158265495575
qsttag_deliver_message_to_prisoner_lord = 504403158265495576
qsttag_duel_for_lady = 504403158265495577
qsttag_duel_courtship_rival = 504403158265495578
qsttag_duel_avenge_insult = 504403158265495579
qsttag_move_cattle_herd = 504403158265495580
qsttag_escort_merchant_caravan = 504403158265495581
qsttag_deliver_wine = 504403158265495582
qsttag_troublesome_bandits = 504403158265495583
qsttag_kidnapped_girl = 504403158265495584
qsttag_persuade_lords_to_make_peace = 504403158265495585
qsttag_deal_with_looters = 504403158265495586
qsttag_deal_with_night_bandits = 504403158265495587
qsttag_deliver_grain = 504403158265495588
qsttag_deliver_cattle = 504403158265495589
qsttag_train_peasants_against_bandits = 504403158265495590
qsttag_eliminate_bandits_infesting_village = 504403158265495591
qsttag_visit_lady = 504403158265495592
qsttag_formal_marriage_proposal = 504403158265495593
qsttag_obtain_liege_blessing = 504403158265495594
qsttag_wed_betrothed = 504403158265495595
qsttag_wed_betrothed_female = 504403158265495596
qsttag_join_faction = 504403158265495597
qsttag_rebel_against_kingdom = 504403158265495598
qsttag_consult_with_minister = 504403158265495599
qsttag_organize_feast = 504403158265495600
qsttag_resolve_dispute = 504403158265495601
qsttag_offer_gift = 504403158265495602
qsttag_denounce_lord = 504403158265495603
qsttag_intrigue_against_lord = 504403158265495604
qsttag_track_down_bandits = 504403158265495605
qsttag_track_down_provocateurs = 504403158265495606
qsttag_retaliate_for_border_incident = 504403158265495607
qsttag_raid_caravan_to_start_war = 504403158265495608
qsttag_cause_provocation = 504403158265495609
qsttag_rescue_prisoner = 504403158265495610
qsttag_destroy_bandit_lair = 504403158265495611
qsttag_blank_quest_2 = 504403158265495612
qsttag_blank_quest_3 = 504403158265495613
qsttag_blank_quest_4 = 504403158265495614
qsttag_blank_quest_5 = 504403158265495615
qsttag_blank_quest_6 = 504403158265495616
qsttag_blank_quest_7 = 504403158265495617
qsttag_blank_quest_8 = 504403158265495618
qsttag_blank_quest_9 = 504403158265495619
qsttag_blank_quest_10 = 504403158265495620
qsttag_blank_quest_11 = 504403158265495621
qsttag_blank_quest_12 = 504403158265495622
qsttag_blank_quest_13 = 504403158265495623
qsttag_blank_quest_14 = 504403158265495624
qsttag_blank_quest_15 = 504403158265495625
qsttag_blank_quest_16 = 504403158265495626
qsttag_blank_quest_17 = 504403158265495627
qsttag_blank_quest_18 = 504403158265495628
qsttag_blank_quest_19 = 504403158265495629
qsttag_blank_quest_20 = 504403158265495630
qsttag_blank_quest_21 = 504403158265495631
qsttag_blank_quest_22 = 504403158265495632
qsttag_blank_quest_23 = 504403158265495633
qsttag_blank_quest_24 = 504403158265495634
qsttag_blank_quest_25 = 504403158265495635
qsttag_blank_quest_26 = 504403158265495636
qsttag_blank_quest_27 = 504403158265495637
qsttag_collect_men = 504403158265495638
qsttag_learn_where_merchant_brother_is = 504403158265495639
qsttag_save_relative_of_merchant = 504403158265495640
qsttag_save_town_from_bandits = 504403158265495641
qsttag_quests_end = 504403158265495642





# -*- coding: utf-8 -*-

## 和python原生相关的工具类功能

def mergeList(*lists):
    '''
        合并多个列表为一个列表
    :param lists:
    :return:
    '''
    result = []
    for list in lists:
        result.extend(list)
    return result


def mergeDict(*dicts):
    '''
        合并多个字典为一个字典
    :param dicts:
    :return:
    '''
    res = {}
    for dict in dicts:
        res.update(dict)
    return res


if __name__ == '__main__':
    dict1 = {"name": "张三"}
    dict2 = {"age": 18}
    dict3 = {"address": 18}

    print(mergeDict(dict1, dict2,dict3))


# ## 模糊匹配类型
# class LikeType():
#     none  = 1
#     left     = 2
#     right   = 3
#     center    = 4
#
#
#
# ## 在数据对比之前作预处理，
# class PreHandle():
#     ## 不做任何处理
#     none = 1
#     ## 将所有特殊符号转换成下划线，可以参考方法：process_common.py文件的convert_to_identifier_with_no_lowercase
#     convertToIdentifier = 2

def convertToIdentifierWithNoLowercase(s0):
    s1 = s0.replace(" ", "_")
    s1 = s1.replace("'", "_")
    s1 = s1.replace("`", "_")
    s1 = s1.replace("(", "_")
    s1 = s1.replace(")", "_")
    s1 = s1.replace("-", "_")
    s1 = s1.replace(",", "_")
    s1 = s1.replace("|", "_")
    s1 = s1.replace("/", "_")
    s1 = s1.replace(".", "_")
    s1 = s1.replace("?", "_")
    s1 = s1.replace("{", "_")
    s1 = s1.replace("}", "_")
    s1 = s1.replace("[", "_")
    s1 = s1.replace("]", "_")
    s1 = s1.replace("!", "_")
    s1 = s1.replace(":", "_")
    s1 = s1.replace("\t", "_")  # Tab
    return s1




menu_start_game_0 = 0
menu_start_phase_2 = 1
menu_start_game_3 = 2
menu_tutorial = 3
menu_reports = 4
menu_custom_battle_scene = 5
menu_custom_battle_end = 6
menu_start_game_1 = 7
menu_start_character_1 = 8
menu_start_character_2 = 9
menu_start_character_3 = 10
menu_start_character_4 = 11
menu_choose_skill = 12
menu_past_life_explanation = 13
menu_auto_return = 14
menu_morale_report = 15
menu_courtship_relations = 16
menu_lord_relations = 17
menu_companion_report = 18
menu_faction_orders = 19
menu_character_report = 20
menu_party_size_report = 21
menu_faction_relations_report = 22
menu_camp = 23
menu_camp_cheat = 24
menu_cheat_find_item = 25
menu_cheat_change_weather = 26
menu_camp_action = 27
menu_camp_recruit_prisoners = 28
menu_camp_no_prisoners = 29
menu_camp_action_read_book = 30
menu_camp_action_read_book_start = 31
menu_retirement_verify = 32
menu_end_game = 33
menu_cattle_herd = 34
menu_cattle_herd_kill = 35
menu_cattle_herd_kill_end = 36
menu_arena_duel_fight = 37
menu_arena_duel_conclusion = 38
menu_simple_encounter = 39
menu_encounter_retreat_confirm = 40
menu_encounter_retreat = 41
menu_order_attack_begin = 42
menu_order_attack_2 = 43
menu_battle_debrief = 44
menu_total_victory = 45
menu_enemy_slipped_away = 46
menu_total_defeat = 47
menu_permanent_damage = 48
menu_pre_join = 49
menu_join_battle = 50
menu_join_order_attack = 51
menu_zendar = 52
menu_salt_mine = 53
menu_four_ways_inn = 54
menu_test_scene = 55
menu_battlefields = 56
menu_dhorak_keep = 57
menu_join_siege_outside = 58
menu_cut_siege_without_fight = 59
menu_besiegers_camp_with_allies = 60
menu_castle_outside = 61
menu_castle_guard = 62
menu_castle_entry_granted = 63
menu_castle_entry_denied = 64
menu_castle_meeting = 65
menu_castle_meeting_selected = 66
menu_castle_besiege = 67
menu_siege_attack_meets_sally = 68
menu_castle_besiege_inner_battle = 69
menu_construct_ladders = 70
menu_construct_siege_tower = 71
menu_castle_attack_walls_simulate = 72
menu_castle_attack_walls_with_allies_simulate = 73
menu_castle_taken_by_friends = 74
menu_castle_taken = 75
menu_castle_taken_2 = 76
menu_requested_castle_granted_to_player = 77
menu_requested_castle_granted_to_player_husband = 78
menu_requested_castle_granted_to_another = 79
menu_requested_castle_granted_to_another_female = 80
menu_leave_faction = 81
menu_give_center_to_player = 82
menu_give_center_to_player_2 = 83
menu_oath_fulfilled = 84
menu_siege_started_defender = 85
menu_siege_join_defense = 86
menu_enter_your_own_castle = 87
menu_village = 88
menu_village_hostile_action = 89
menu_recruit_volunteers = 90
menu_village_hunt_down_fugitive_defeated = 91
menu_village_infest_bandits_result = 92
menu_village_infestation_removed = 93
menu_center_manage = 94
menu_center_improve = 95
menu_town_bandits_failed = 96
menu_town_bandits_succeeded = 97
menu_village_steal_cattle_confirm = 98
menu_village_steal_cattle = 99
menu_village_take_food_confirm = 100
menu_village_take_food = 101
menu_village_start_attack = 102
menu_village_loot_no_resist = 103
menu_village_loot_complete = 104
menu_village_loot_defeat = 105
menu_village_loot_continue = 106
menu_close = 107
menu_town = 108
menu_cannot_enter_court = 109
menu_lady_visit = 110
menu_town_tournament_lost = 111
menu_town_tournament_won = 112
menu_town_tournament_won_by_another = 113
menu_town_tournament = 114
menu_tournament_withdraw_verify = 115
menu_tournament_bet = 116
menu_tournament_bet_confirm = 117
menu_tournament_participants = 118
menu_collect_taxes = 119
menu_collect_taxes_complete = 120
menu_collect_taxes_rebels_killed = 121
menu_collect_taxes_failed = 122
menu_collect_taxes_revolt_warning = 123
menu_collect_taxes_revolt = 124
menu_train_peasants_against_bandits = 125
menu_train_peasants_against_bandits_ready = 126
menu_train_peasants_against_bandits_training_result = 127
menu_train_peasants_against_bandits_attack = 128
menu_train_peasants_against_bandits_attack_result = 129
menu_train_peasants_against_bandits_success = 130
menu_disembark = 131
menu_ship_reembark = 132
menu_center_reports = 133
menu_price_and_production = 134
menu_town_trade = 135
menu_town_trade_assessment_begin = 136
menu_town_trade_assessment = 137
menu_sneak_into_town_suceeded = 138
menu_sneak_into_town_caught = 139
menu_sneak_into_town_caught_dispersed_guards = 140
menu_sneak_into_town_caught_ran_away = 141
menu_enemy_offer_ransom_for_prisoner = 142
menu_training_ground = 143
menu_training_ground_selection_details_melee_1 = 144
menu_training_ground_selection_details_melee_2 = 145
menu_training_ground_selection_details_mounted = 146
menu_training_ground_selection_details_ranged_1 = 147
menu_training_ground_selection_details_ranged_2 = 148
menu_training_ground_description = 149
menu_training_ground_training_result = 150
menu_marshall_selection_candidate_ask = 151
menu_captivity_avoid_wilderness = 152
menu_captivity_start_wilderness = 153
menu_captivity_start_wilderness_surrender = 154
menu_captivity_start_wilderness_defeat = 155
menu_captivity_start_castle_surrender = 156
menu_captivity_start_castle_defeat = 157
menu_captivity_start_under_siege_defeat = 158
menu_captivity_wilderness_taken_prisoner = 159
menu_captivity_wilderness_check = 160
menu_captivity_end_wilderness_escape = 161
menu_captivity_castle_taken_prisoner = 162
menu_captivity_rescue_lord_taken_prisoner = 163
menu_captivity_castle_check = 164
menu_captivity_end_exchanged_with_prisoner = 165
menu_captivity_end_propose_ransom = 166
menu_captivity_castle_remain = 167
menu_kingdom_army_quest_report_to_army = 168
menu_kingdom_army_quest_messenger = 169
menu_kingdom_army_quest_join_siege_order = 170
menu_kingdom_army_follow_failed = 171
menu_invite_player_to_faction_without_center = 172
menu_invite_player_to_faction = 173
menu_invite_player_to_faction_accepted = 174
menu_question_peace_offer = 175
menu_notification_truce_expired = 176
menu_notification_feast_quest_expired = 177
menu_notification_sortie_possible = 178
menu_notification_casus_belli_expired = 179
menu_notification_lord_defects = 180
menu_notification_treason_indictment = 181
menu_notification_border_incident = 182
menu_notification_player_faction_active = 183
menu_minister_confirm = 184
menu_notification_court_lost = 185
menu_notification_player_faction_deactive = 186
menu_notification_player_wedding_day = 187
menu_notification_player_kingdom_holds_feast = 188
menu_notification_center_under_siege = 189
menu_notification_village_raided = 190
menu_notification_village_raid_started = 191
menu_notification_one_faction_left = 192
menu_notification_oath_renounced_faction_defeated = 193
menu_notification_center_lost = 194
menu_notification_troop_left_players_faction = 195
menu_notification_troop_joined_players_faction = 196
menu_notification_war_declared = 197
menu_notification_peace_declared = 198
menu_notification_faction_defeated = 199
menu_notification_rebels_switched_to_faction = 200
menu_notification_player_should_consult = 201
menu_notification_player_feast_in_progress = 202
menu_notification_lady_requests_visit = 203
menu_garden = 204
menu_kill_local_merchant_begin = 205
menu_debug_alert_from_s65 = 206
menu_auto_return_to_map = 207
menu_bandit_lair = 208
menu_notification_player_faction_political_issue_resolved = 209
menu_notification_player_faction_political_issue_resolved_for_player = 210
menu_start_phase_2_5 = 211
menu_start_phase_3 = 212
menu_start_phase_4 = 213
menu_lost_tavern_duel = 214
menu_establish_court = 215
menu_notification_relieved_as_marshal = 216
menu_survival_wild = 217
menu_land_manage_options = 218
menu_farmer_change_seed_list = 219



# -*- coding: utf-8 -*-
# Point export_dir to the folder you will be keeping your module
# Make sure you use forward slashes (/) and NOT backward slashes (\)
# import os
# path = os.path.abspath("../..");

## 编译到项目目录
##export_dir = path + "/build/"

# ## 编译到mod中
# export_dir = "D:\\Program Files (x86)\\Mount&Blade Warband\\Modules\\Native\\"
#
# ## 脚本所在目录
# src_dir = path + "/src/smart_module/module_system/"
